import tensorflow as tf
from keras import backend as K
from keras.engine.topology import Layer
import numpy as np

class Anygram(Layer):
    '''
    Keras custom layer for any-grams
    '''

    def __init__(self, **kwargs):
        '''
        '''
        
        super(Anygram, self).__init__(**kwargs)
        
    
    
    def build(self, input_shape):
        '''
        '''
        
        ############
        # Create a trainable weight variable for this layer.
        #self.kernel = self.add_weight(name = 'kernel', 
        #                             shape = (input_shape[1], self.output_dim),
        #                              initializer = 'uniform',
        #                              trainable = True)
        ###########
        
        super(Anygram, self).build(input_shape)
        
    
    
    def call(self, x):
        '''
        '''
        
        ############
        #return K.dot(x, self.kernel)
        ############
        
        l = x.shape[1]          # input sequence length
        
        # generating weight matrix to filter out tokens not in the n-grams:
        # 
        # For each n-gram, there is a weight vector of size equal to the sequence length, where the elements 
        # represent the tokens. The weights are binary values, 1 for tokens in the n-gram, 0 for tokens not in 
        # the n-gram. For example, for a sequence of 4 words {s4,s3,s2,s1} (e.g. I eat banana.), the weight 
        # vector [0, 0, 0, 1] generates the unigram s1, [1, 1, 1, 0] generates the trigram s4s3s2 and [1, 1, 0, 1] 
        # is not valid as s4s3s1 is not a valid n-gram.
        # 
        # It's clear that these vectors resemble binary numbers. For example, 111 (decimal 7) can be used to 
        # generate the rightmost 3-gram (s3s2s1) and 1110 (decimal 14, i.e. previous 7 times 2) to generate the
        # next 3-grams (s4s3s2). So, the formula for generating these binary numbers is 2^i*(2^n-1) where i is 
        # the index of the n-gram and is in the range [0, l-n], l being the length of the sequence and n the 
        # order of the n-gram. For example in a sequence of length 5 (l=5), the rightmost (i=0) 2-gram (n=2) will 
        # be represented by 2^0*(2^2-1)=3 which is binary 11. The 3rd 3-gram from right (i=2) will be represented 
        # by 2^2(2^3-1)=28 which is binary 11100.
        #
        # The following is how this is implemented. The output is a tensor with shape (bs, l, l, es), where the bs 
        # is the batch size, l the sequence length, and es the embedding vector size for each token/word.
        
        w = tf.convert_to_tensor(np.array([[np.frombuffer(np.binary_repr(2**i*(2**n-1)).rjust(l, '0'), dtype=np.uint8) - ord('0')  for i in range(l - n + 1)] +
                                           [np.zeros(l, dtype=int) for j in range(n - 1)]
                                           for n in range(1, l + 1)])
                                , dtype=tf.float32)
        
        # generating the any-grams
        
        return K.permute_dimensions(K.dot(K.permute_dimensions(x, (0, 2, 1)), 
                                          K.transpose(w)), 
                                    (0, 2, 3, 1))
        
    
    
    def compute_output_shape(self, input_shape):
        '''
        '''
        
        return (input_shape[0], input_shape[1], input_shape[1], input_shape[2])
        
    
    
