#! /usr/bin/python

## This script extracts the N top-ranked words from the output of 
## weka.attributeSelection.InfoGainAttributeEval, which ranks features
## based on their information gain.
##
## Current version: 1.1
##
## Changes:
## version 1.1:
## - The main part of the code was separated out into wekaigaeproc.py module
##   to be reused elsewhere.
## This script calls extractTopFeatures() functions from wekaigaeproc.py
## to print the requested number of topmost features ranked by weka 
## function weka.attributeSelection.InfoGainAttributeEval in the input 
## file.
##

import sys, optparse, weka


##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv
 
 parser = optparse.OptionParser(usage="%prog <WEKA IGAE OUTPUT FILE> <NUMBER OF WORDS TO EXTRACT>" + 
                                      "\nExtracts the N top-ranked words from the output of weka.attributeSelection.InfoGainAttributeEval.", version="%prog 1.1")

 (opts, posArgs) = parser.parse_args()
 
 ## checking arguments
 
 if len(posArgs) < 2:
  parser.error("At least 2 arguments are required!")
  
 if not posArgs[1].isdigit():
  parser.error("The second argument should be a number!")
 else:
  vTopNum = int(posArgs[1])

 # calling function to extract and print top IG-based ranked features
 weka.extractTopIGFeatures(posArgs[0], vTopNum, True)


##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())




