__author__ = 'rszk'
# -*- coding: utf-8 -*-

"""	
	This module defines functions and classes for linear regression algorithm.
	
	Version 0.1							    (12-Nov-2015 to 14-Dec-2015)
	- LineaReg is added.
	
"""


import sys
import numpy as np


class LineaReg:
	'''
	Class for linear regression algorithm
	'''
	
	
	def __init__(self, pTrainSet = None, pInitialWeights = "zero", pLRate = 0.1, pMaxIter = 0):
		'''
		Constructor
		'''
		
		self.trainSet = pTrainSet
		
		# weights initialization method
		self._initialWeights = pInitialWeights.lower()
		
		# weight vector
		self.weights = []
		
		# setting learning rate
		self.lRate = pLRate
		
		# maximum number of iterations to converge; 0 means no maximum.
		self.maxIter = pMaxIter
		
	
	
	def loadTrainSet(self, pTrainSet):
		'''
		Loads training data set which is a data.Dataset object
		'''
		
		self.trainSet = pTrainSet
		
		# reset all parameters trained with the previous data set if applies
		
	
	
	def train(self, pGauge = True):
		'''
		Trains a linear regression model
		'''
		
		# initializing weights
		self._initWeights()
		vIter = 0
		
		# learning iterations (to update weights)
		while True:
			# calculating gradient
			vaGradient = self.calcGradient()
			#print self.weights
			#print vaGradient
			self.weights = np.subtract(self.weights, np.multiply(self.lRate / self.trainSet.size, vaGradient))
			#print self.weights
			#print
			#raw_input()
			
			vIter += 1 
			if pGauge:
				sys.stdout.write('.')
			
			# Note: if maxIter is set to 0, it won't stop until the weights converge (i.e. the gradient is all 0)
			if vIter == self.maxIter or not np.any(vaGradient):
				break
			else:
				continue
		
		if pGauge:
			print 
		
		#print "Gradient:", vaGradient.tolist()
		#print "Weights:", self.weights.tolist()
		#print
		
		return vIter
		
	
	
	def plotRegLine(self, pOutFilename):
		'''
		Plots the regression line for one dimensional data
		
		The plot will be saved into the given output file or be shown on display if no file is given. The extension of
		the file determines the format which must be one of the following according to matplotlib:
		   eps, jpeg, jpg, pdf, pgf, png, ps, raw, rgba, svg, svgz, tif, tiff
		'''
		
		if self.trainSet.dimension > 1:
			print "The regression line can only be plotted for one dimensional data!"
			return
		
		import matplotlib.pyplot as plt
		
		vFig = plt.figure()
		vAxe =vFig.add_subplot(111)
		
		# getting feature values
		vFValues = self.trainSet.features[1].getValues()
		
		# plotting the data
		vAxe.plot(vFValues, self.trainSet.getTargets(), 'bo')
		
		# plotting the regression line
		vAxe.plot(vFValues, [self._calculateTarget([1, x]) for x in vFValues])
		
		if pOutFilename:
			vFig.savefig(pOutFilename)
		else:
			vFig.show()
		
	
	
	def _initWeights(self):
		'''
		Initializes weights using the method specified in the settings
		'''
		
		if self._initialWeights.startswith("zero"):
			self.weights = np.zeros(1 + self.trainSet.dimension)
		elif self._initialWeights.startswith("rand"):
			self.weights = np.random.rand(1 + self.trainSet.dimension)
		
	
	
	def calcCost(self):
		'''
		Calculates and returns the cost over the training data with the current weight vector
		 
		This is in fact the cost, loss or objective function.
		'''
		
		vSum = 0
		
		for vInstance in self.trainSet.instances:
			vInputVector = [1] + vInstance.getFeatureValueVector()
			vSum += (self._calculateTarget(pInputVector = vInputVector) - vInstance.target) ** 2
		
		return vSum / (2.0 * self.trainSet.size) 
		
	
	
	def _calculateTarget(self, pInputVector):
		'''
		Calculates the output using the current weights and instance vector
		'''
		
		return np.dot(self.weights, pInputVector)
		
	
	
	def calcGradient(self):
		'''
		Calculates and returns the gradient over the training data with the current weight vector
		
		The returned value is a vector with its dimension equal to the number of features (training data dimension) plus
		1 for the intercept.
		
		Algorithm:
		
		The algorithm uses vectorization for efficient computation. There are two ways to do the vectorization:
		
		 1- The first method is more or less based on the tutorial given by Andrew Ng for gradient descent for linear regression
		    (see http://openclassroom.stanford.edu/MainFolder/VideoPage.php?course=MachineLearning&video=02.6-LinearRegressionI-VectorizedImplementation).
		    In this method, gradient vector G is calculated as: 
		    
		    G = e(1) . X(1) + 
		        e(2) . X(2) + 
		        ...         +
		        e(m) . X(m)
		    
		    where e is the prediction error for a specific instance (predicted target using current weights minus the 
		    gold target) and X is feature value vector (including 1 for intercept) for that specific instance.
		    
		 2- The second method used here can be said is more vectorized than the first method (in fact matricized), and the 
		    gradient vector is calculated as:
		    
		    G = FT . ET
		    
		    where FT is the transposition of the feature value matrix (including 1 for intercepts) as follows:
		    
		    |   1       1       ...     1       |
		    |   x(1,1)  x(2,1)  ...     x(m,1)  |
		    |   x(1,2)  x(2,2)  ...     x(m,2)  |
		    |   ...                             |
		    |   x(1,n)  x(2,n)  ...     x(m,n)  |
		    
		    where m is the training set size and n is the number of features.
		    
		    And ET is the transposition of the error vector as follows:
		    
		    |   e(1)  |
		    |   e(2)  |
		    |   ...   |
		    |   e(m)  |
		    
		    where e is the prediction error for a specific instance as seen before and m is the training set size again.
		    This is calculated as the dot product of the weight vector with a -1 as its last element and the transposition
		    of the data matrix. The data matrix is the transposed feature value with an additional last row containing the 
		    gold target values. The formula for ET is therefore:
		    
		    |   w(0)   w(1)   ...   w(n)   -1   | . |   x(1, 1)   x(2,1)   ...   x(m,1)  |
		                                            |   x(1, 2)   x(2,2)   ...   x(m,2)  |
		                                            |   ...                              |
		                                            |   x(1, n)   x(2,n)   ...   x(m,n)  |
		                                            |   y(1)      y(2)    ...    y(m)    |
		    
		    As can be seen, the insertion of -1 and target vector at the end of the two matrices will result in the 
		    subtraction of the target values from predicted values in the output dot product matrix. 
		    
		    We use this second method here to calculate the gradient.
		
		    The gradient calculated here will be used to calculate new weights as:
		 
		    W = W - (r / m) . G
		'''
		
		vaFMatrixT = np.concatenate(([np.ones(self.trainSet.size)],
		                             self.trainSet.getFeatureMatrixT()))
		vaErrT = np.dot(np.append(self.weights, [-1]),
		                np.concatenate((vaFMatrixT, [self.trainSet.getTargets()])))
		
		vaGradient = np.dot(vaFMatrixT, vaErrT)
		
		return vaGradient
		
	
	
	def predict(self, pInstance):
		'''
		Given a data instance, predicts its target with the learnt weights
		'''
		
		return self._calculateTarget(pInputVector = [1] + pInstance.getFeatureValueVector())
		
	
	
