__author__ = 'rszk'
#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""	
	NOTE: CONVERT THINGS TO NUMPY FOR EFFICIENCY
	
	This module defines functions and classes for Perceptron algorithm.
	
	Version 0.1								(13-Oct-2015 to 15-Oct-2015)
	- Perceptron is added.
	
"""

import numpy as np

class Perceptron:
	'''
	Class for Perceptron algorithm
	'''
	
	
	def __init__(self, pTrainSet = None, pInitialWeights = "zero", pflgShuffleTrainSet = False, pLRate = 0.1, pMaxIter = 0, pErrThreshold = 0):
		'''
		Constructor
		
		The input training set is a data.Dataset object.
		
		If the maximum iteration (epoch) number is set to 0, the training will continue until the error becomes 0 or forever. 
		In other words, 0 (default) means no maximum.
		
		The weights can be initialized in a number of ways:
		- zero (all zero)
		- random
		'''
		
		self.trainSet = pTrainSet
		
		# weights initialization method
		self._initialWeights = pInitialWeights.lower()
		
		# weight vector
		self.weights = []
		
		# whether or not shuffle the training set at each iteration
		self._shuffleTrainSet = pflgShuffleTrainSet
		
		# setting learning rate
		self.lRate = pLRate
		
		# maximum number of iterations to converge; 0 means no maximum.
		self.maxIter = pMaxIter
		
		# error threshold for stopping the training: stop if the error on the training set is less than or equal to this   
		self.errThreshold = pErrThreshold
		
	
	
	def loadTrainSet(self, pTrainSet):
		'''
		Loads training data set which is a data.Dataset object
		'''
		
		self.trainSet = pTrainSet
		
		# clearing the existing weights
		self.weights = []
		
	
	
	def train(self):
		'''
		Trains a Perceptron model (i.e. learns weights) and returns the number of iterations
		'''
		
		vDataDimension = self.trainSet.dimension
		
		# initializing weights
		self._initWeights()
		#print self.weights
		vIter = 0
		
		# learning iterations (to update weights)
		while True:
			vlErrors = []
			
			if self._shuffleTrainSet:
				pass; ########### implement self.trainSet.shuffle() 
			
			# iterating through training instances to calculate the instance errors with the new weights
			for vInstance in self.trainSet.instances:
				# calculating the output with the current weights and instance vector
				vInputVector = [1] + vInstance.getFeatureValueVector()
				vOutput = self._calculateOutput(vInputVector)
				print 'output: %s' % vOutput
				
				# calculating the instance error
				vIError = vInstance.target - vOutput
				print "error: %s" % vIError
				vlErrors.append(vIError)
				
				# updating weights
				self.weights = self._updateWeights(pError = vIError, pInputVector = vInputVector)
				print "weights: %s\n" % self.weights
			
			# calculating iteration error
			## NOTE: I have not seen using the absolute value of the instance errors anywhere else but it seems to be 
			## working better (more accurate)
			#vError = sum(vlErrors) / self.trainSet.size
			vError = sum([abs(e) for e in vlErrors]) / self.trainSet.size
			print "global error: %s, %s\n" % (vlErrors, vError)
			
			vIter += 1 
			
			# Note: if maxIter is set to 0, it won't stop until error becomes 0.
			if vIter == self.maxIter or vError <= self.errThreshold:
				break
			else:
				continue
		
		return vIter
		
	
	
	def _initWeights(self):
		'''
		Initializes weights using the method specified in the settings
		'''
		
		if self._initialWeights.startswith("zero"):
			self.weights = np.zeros(1 + self.trainSet.dimension)
		elif self._initialWeights.startswith("rand"):
			self.weights = np.random.rand(1 + self.trainSet.dimension)
		
	
	
	def _calculateOutput(self, pInputVector):
		'''
		Calculates the output using the current weights and instance vector
		'''
		
		return self.stepFunc(np.dot(self.weights, pInputVector))
		
	
	
	def classify(self, pInstance):
		'''
		Given a data instance, classifies it with the learnt weights
		'''
		
		return self._calculateOutput(pInputVector = [1] + pInstance.getFeatureValueVector())
		
	
	
	def stepFunc(self, pInput):
		'''
		Step function
		'''
		
		if pInput >= 0:
			return 1
		else:
			return -1
		
	
	
	def _updateWeights(self, pError, pInputVector):
		'''
		Updates the weights based on the instance error value provided, the learning rate
		'''
		
		return [w + (self.lRate * pError * iv) for w, iv in zip(self.weights, pInputVector)]
		
	
	

