#! /usr/bin/python

"""
	This module provides classes to generate features vectors for SVM.
	
	Version 0.5												(26-May-2013)
	- The FVGen and Feature classed are moved to a new module fvg.py which 
	  aims at generalizing the classes to a broader range of machine learning
	  algorithms. The SVMFVGen is now inherited from fvg.FVGen.
	- genFeatureVectors() overrides fvg.FVGen.genFeatureVectors()
	
	Version 0.4												(26-May-2013)
	- Multi-value features (e.g. n-grams) are supported.
	
	Version 0.3												(09-May-2013)
	- Feature.params is added to support feature-specific settings.
	
	Version 0.2												(08-May-2013)
	- Rounding was added to the global configuration.
	- Feature index is added to FVGen.features.
	
	Version 0.1												(07-May-2013)
	- FVGen is added.
	- Feature is added.
	
"""

from utils import util
import fvg
import sys, yaml, random

class SVMFVGen(fvg.FVGen):
	'''
	Feature vector generator for SVM
	'''
	
	
	def __init__(self, pConfig):
		'''
		Constructor
		'''
		
		fvg.FVGen.__init__(self, pConfig)
		
	
	
	def genFeatureVectors(self):
		'''
		Generates and returns feature vectors and feature configuration.
		
		A feature vector is a string in the classic SVM input format, e.g.:
		    1:0.01 2:0.875 3:0.2
		'''
		
		vldFVectors, vConfig = super(SVMFVGen, self).genFeatureVectors()
		
		vlFVectors = []
		for vdFV in vldFVectors:
			vlFVectors.append('\t'.join(["%s:%s" % (no, self._round(value[1])) for no, value in vdFV.iteritems()]))
		
		return vlFVectors, vConfig
		
	
	
