#! /usr/bin/python

"""
	This module defines functions related to Weka toolkit, such as 
	generating input files for Weka or processing its output files.
	
	Version: 1.1
	- extractClassifierPreds() is changed to use default prediction 
	column instead of getting it as argument.
	
	Version: 1.0
"""

import sys, optparse


##----------------------------------------------------------------------
## Extracts top N features from pIGAEFileName and prints
## any of them to stdout if requested. 
##
## This function was primarily written to extract words having the highest
## information gain with respect to a set of best/worst parses, to be 
## used as lexical features in parser accuracy prediction.
def extractTopIGFeatures(pIGAEFileName, pTopNum, pflgPrint):
 vlTopFeatures = []
 
 # opening weka IGAE output file
 try:
  vIGAEFile  = open(pIGAEFileName, 'r')
 except IOError:
  sys.exit('Can\'t open weka IGAE output file: ' + pIGAEFileName)


 # ignoring the header
 vLine=vIGAEFile.readline()
 while vLine:
  ## we suppose the following is the signal of ranking list start
  if vLine[:18].lower() == "ranked attributes:":
   break
  else:
   vLine=vIGAEFile.readline()

 vCntr = 0
 # processing ranking list part of the file
 vLine=vIGAEFile.readline()
 while vLine:
  ## we stop if an empty line is faced or the request number of words are
  ## extracted, whichever reaches first.
  if (vCntr == pTopNum):
   break
  elif len(vLine.strip('\n')) == 0:
   sys.stderr.write("Warning: size of ranking list was smaller than the number of words requested\n")
   break
  else:
   vLineSplit = vLine.split()

  # the 3rd column is supposed to be the feature
  vlTopFeatures.append(vLineSplit[2].strip())
  if pflgPrint:
   print vlTopFeatures[-1]
  
  vCntr += 1

  # read the next line
  vLine=vIGAEFile.readline()
  
 vIGAEFile.close()
 return vlTopFeatures


def extractPredictions(pPredFileName, pflgPrint):
	'''
	Extracts the predictions from weka output file, and optionally prints
	it to stdout.
	'''
	
	vlPreds = []
	
	# opening weka classifier output file
	try:
		vfPredFile  = open(pPredFileName, 'r')
	except IOError:
		sys.exit('Can\'t open weka classifier output file: ' + pPredFileName)
	
	# ignoring the header
	vLine=vfPredFile.readline()
	while vLine:
	## we suppose the following is the signal of predictions start
		if vLine.lstrip()[:5].lower() == "inst#":
			break
		else:
			vLine=vfPredFile.readline()
	
	# processing ranking list part of the file
	vLine=vfPredFile.readline()
	while vLine:
	## we stop if an empty line is faced
		if len(vLine.strip('\n')) == 0:
			break
		else:
			vLineSplit = vLine.split()
		
		# prediction is supposed to be in 3rd column
		vlPreds.append(vLineSplit[2].strip())
		if pflgPrint:
			print vlPreds[-1]
	
		# read the next line
		vLine=vfPredFile.readline()
	
	vfPredFile.close()
	return vlPreds
	

