#! /usr/bin/python

## This script combines the outputs of several MT systems by selecting 
## the best output based on sentence-by-sentence translation scores. It 
## also outputs the scores combination along with the seleted system for
## each sentence.
##
## The input is one translation output file and one corresponding score 
## file per MT system.
##
## The number oand names of systems should be input as well to the script.
##
## In the cases of score ties, the scripts offers two ways to break them:
## selecting the first best in the order system names, translations, and
## scores are provided for the script and selecting one randomly.
##
## Current version: 1.0


import sys, optparse, mtscc


##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv

 parser = optparse.OptionParser(usage="%prog <NUMBER OF SYSTEMS> <SYSTEM 1 [SYSTEM 2 [..]]> <TRANSLATION FILE 1 [TRANSLATION FILE 2 [..]]> <SCORE FILE 1 [SCORE FILE 2 [..]]> <SELECTION OUTPUT FILE NAME> [OPTIONS]" +
                                      "\nExtract one translation per source sentence from a set of parallel translation files based on input file containing the selections per source sentence.", version="%prog 1.0")

 parser.add_option("-t", "--tiebreak", help="the tie breaking method (o: system order, r: random)", metavar="TIEBREAK", dest="tieBreak", default="a", action="store")

 (opts, posArgs) = parser.parse_args()


 # processing arguments

 if len(posArgs) < 5:
  parser.error("At least 5 arguments are required!")
 
 vSysCnt = int(posArgs[0])
 vlSysNames = posArgs[1: 1 + vSysCnt]
 vlTransFileNames = posArgs[1 + vSysCnt: 1 + vSysCnt * 2]
 vlScoreFileNames = posArgs[1 + vSysCnt * 2: 1 + vSysCnt * 3]
 vSelectionFileName = posArgs[1 + vSysCnt * 3]
 
 # combining outputs and writing selection file
 mtscc.combineOutputs(vlSysNames, vlTransFileNames, vlScoreFileNames, opts.tieBreak, vSelectionFileName)

 
##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())

