#! /usr/bin/python

## This script extracts sentence-level features from juxtaposition file
## of MT system outputs. 
##
## It calls loadJuxta() and extractSentenceFeaturesJux() from mtscc.py 
## module to load the juxtaposition from file and extract features. It 
## calls other feature-specific functions from mtscc.py to extract ad-hoc
## features from those sentence-level feature. These features are called
## ad-hoc because there is no plan for combination based on methods like
## classification, and they are extracted mainly for analysis purposes.
##
## Version 1.1
## - Extracting average length ratios is added.
##
## Current version 1.0
## - Currently, comma-separated list of feature value are written into 
##   output and no classification format is used.
##


import sys, optparse, mtscc


##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv
 
 parser = optparse.OptionParser(usage="%prog <JUXTAPOSITION FILE> " + 
                                      "\nThis script extracts sentence-level features from juxtaposition file of MT system outputs.", 
                                      version="%prog 1.1")
 parser.add_option("-p", "--parse", help="parse tree file of the source sentences", metavar="PARSE FILE", dest="parseFileName", action="store")

 (opts, posArgs) = parser.parse_args()
 
 # processing arguments
 
 if len(posArgs) < 1:
  parser.error("At least 1 arguments are required!")

 
 # openning source side parse tree file if provided 
 if opts.parseFileName != None:
  try:
   vfParse = open(opts.parseFileName)
  except IOError:
   sys.exit("Can't open source parse file: " + opts.parseFileName)
 
  
 # loading juxtapositions
 vlJuxta = mtscc.loadJuxta(posArgs[0])
 
 # creating output file for sentence-level features
 try:
  vfSntcFeature  = open("sentence-features", 'w')
 except IOError:
  sys.exit("Can't create sentence-level feature file: sentence-features")
 
 # extracting sentence-level features
 vlFeatures = mtscc.extractSentenceFeaturesJux(vlJuxta, True, True, True, True, vfSntcFeature)
 
 ## creating output file for source length feature (associated with average
 ## scores)
 try:
  vfAvgScorePerLenValue  = open("avg-score-per-len-value.min-5", 'w')
 except IOError:
  sys.exit("Can't create output feature file: avg-score-per-len-value.min-5")
 
 # extracting average score per source length for all systems
 mtscc.extractAvgScorePerSrcLen(vlFeatures, 5, True, vfAvgScorePerLenValue)

 ## creating output file for average target to source length ratio of
 ## referrence and system outputs
 try:
  vfAvgLenRatio  = open("avg-len-ratio", 'w')
 except IOError:
  sys.exit("Can't create output feature file: avg-len-ratio")
 
 # extracting average length ratios (for reference plus all systems)
 mtscc.extractAvgLenRatio(vlFeatures, [vSys.sysName for vSys in vlJuxta[0].outputs], True, vfAvgLenRatio)

 vfSntcFeature.close()
 vfAvgScorePerLenValue.close()
 vfAvgLenRatio.close()
 try:
  vfParse.close()
 except NameError:
  pass 
 
##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())

