#! /usr/bin/python

## This script extract one translation per source sentence from a set of
## parallel files containing the output of different translation systems
## based on an input file containing the selections per source sentence.
##
## It expects the translations in files with the name of systems in selection
## file.
##
## Current version: 1.0

from collections import namedtuple
import sys, optparse



##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv

 parser = optparse.OptionParser(usage="%prog <SELECTION FILE> <TRANSLATION DIRECTORY> [OPTIONS]" +
                                      "\nExtract one translation per source sentence from a set of parallel translation files based on input file containing the selections per source sentence.", version="%prog 1.0")

 parser.add_option("-r", "--firstrow", help="line number of first sentence in selection file", metavar="FIRST SELECTION ROW", default=1, dest="firstRow", action="store")
 parser.add_option("-c", "--selectcol", help="column number of selection in selection file", metavar="SELECTION COLUMN NUMBER", default=1, dest="selCol", action="store")
 parser.add_option("-w", "--weka", help="the selection file is Weka classifier output", metavar="IS WEKA", default=1, dest="isWeka", action="store_true")

 (opts, posArgs) = parser.parse_args()


 # processing arguments

 if len(posArgs) < 1:
  parser.error("At least selection file is required!")

 ## the current path is considered the default path containing translation
 ## files
 if len(posArgs) == 1:
  vTransPath = ""
 else:
  vTransPath = posArgs[1]
  if not vTransPath.endswith('/'):
   vTransPath += '/'

 # opening selection file
 try:
  vfSelection  = open(posArgs[0], 'r')
 except IOError:
  sys.exit('Can\'t open selection file: ' + posArgs[0])


 # extracting selections
 vLineCntr = 1
 vlSelections = []
 for line in vfSelection:
  if vLineCntr < int(opts.firstRow):
   vLineCntr += 1
   continue
  elif line.strip() == "":
   break
  else:
   ## In Weka classifier output, class name is in the format of
   ## classno:classname, of which we need classname
   if opts.isWeka:
    vlSelections.append(line.split()[int(opts.selCol) - 1].split(':')[1])
   else:
    vlSelections.append(line.split()[int(opts.selCol) - 1])

   vLineCntr += 1

 vfSelection.close();

 # identifying systems
 vlSystems = [item for item in set(vlSelections)]


 ## loading translations by iterating identified systems
 ## Thranslations are stored in a list of list: one dimentsion representing
 ## system and the other sentence translations
 vlTranslations = []
 for system in vlSystems:
  try:
   vfTrans = open(vTransPath + system, 'r')
   vlTranslations.append(vfTrans.readlines())
  except IOError:
   sys.exit('Can\'t open translation file: ' + vTransPath + '/' + system)
  finally:
   vfTrans.close()

 # extracting translations
 vSentenceCntr = 1
 for selSys in vlSelections:
  print vlTranslations[vlSystems.index(selSys)][vSentenceCntr - 1],
  vSentenceCntr += 1


##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())



