#! /usr/bin/python

## This script juxtuposes outputs of multiple MT systems together with 
## the source and reference, sentence by sentence, for comparsion purposes.
## 
## It uses functions in mtscc.py module for this purpose. The main output
## is the full juxtaposed output. However, it can create juxtaposed output
## of cases with score ties, real sentence ties, and individual system 
## wins. The statistics of these ties and wins can be optionally printed
## to standard output.
##
## Current version: 1.1
## - Writing output was moved to mtscc.py module.


import sys, optparse, mtscc


##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv
 
 parser = optparse.OptionParser(usage="%prog <SOURCE FILE> <REFERENCE FILE> <NUMBER OF SYSTEMS> <SYSTEM 1 NAME [..]> <SYSTEM 1 OUTPUT FILE [..]> <SYSTEM 1 SCORE FILE [..] <OUTPUT DIR> <JUXTAPOSITION FILES PREFIX> [options]" + 
                                      "\nThis script juxtuposes outputs of multiple MT systems together with the source and reference, sentence by sentence, for comparsion purposes.", 
                                      version="%prog 1.1")

 parser.add_option("--scoretiesplit", help="output split of full juxtaposition based on score tie and differences", metavar="SCORE TIES", dest="outScoreTieSplit", action="store_true")
 parser.add_option("--realtiesplit", help="ouput split of full juxtaposition based on real (sentence) tie and differences", metavar="REAL TIES", dest="outRealTieSplit", action="store_true")
 parser.add_option("--winsoutinfix", help="output file infix for split of juxtaposition based on win cases of each individual system", metavar="WINS OUT INFIX", dest="winsInfix", action="store")
 parser.add_option("-s", "--stat", help="print statistics for ties and wins", metavar="PRINT STATS", dest="printStats", action="store_true")
 parser.add_option("-d", "--scoresort", help="direction of sorting scores: (a)scending (e.g. TER) (d)escending", metavar="SCORE SORT DIRECTION", dest="scoreSortDir", default="a", action="store")
 parser.add_option("-t", "--tab", help="insert tab to separate output fiedls", metavar="TABULARIZE", dest="tabularize", action="store_true")

 (opts, posArgs) = parser.parse_args()
 
 # processing arguments
 
 if len(posArgs) < 8:
  parser.error("At least 8 arguments are required!")
  
 vSrcFileName = posArgs[0]
 vRefFileName = posArgs[1]
 vSysCount = int(posArgs[2])
 vlSysNames = posArgs[3: 3 + vSysCount]
 vlSysOutFileNames = posArgs[3 + vSysCount: 3 + vSysCount * 2]
 vlSysOutScoreFileNames = posArgs[3 + vSysCount * 2: 3 + vSysCount * 3]
 vOutputDir = posArgs[3 + vSysCount * 3]
 vJuxtOutFilePrefix = posArgs[4 + vSysCount * 3]
   
  
 # creating the full juxtaposed output file
 vlJuxtaposition = mtscc.juxtaposeOutput(vSrcFileName, vRefFileName, vlSysNames, vlSysOutFileNames, vlSysOutScoreFileNames, False)
 
 
 # writing the outputs

 vFullOutFileName = vOutputDir + '/' + vJuxtOutFilePrefix + ".jux-full"

 if opts.outScoreTieSplit:
  vScoreTieFileName = vOutputDir + '/' + vJuxtOutFilePrefix + '.jux-score-tie'
  vScoreDifFileName = vOutputDir + '/' + vJuxtOutFilePrefix + '.jux-score-dif'

 if opts.outRealTieSplit:
  vRealTieFileName = vOutputDir + '/' + vJuxtOutFilePrefix + '.jux-real-tie'
  vRealDifFileName = vOutputDir + '/' + vJuxtOutFilePrefix + '.jux-real-dif'

 ## creating file name list for individual system wins
 ## - Any-wins: system is winner in any case)
 ## - Solo-wins: system is the only winner)
 vlAnyWinFileNames = []
 vlSoloWinFileNames = []
 if opts.winsInfix != None:
  for vSys in vlSysNames:
   vlAnyWinFileNames.append(vOutputDir + '/' + vSys + '.' + opts.winsInfix + '.jux-all-wins')
   vlSoloWinFileNames.append(vOutputDir + '/' + vSys + '.' + opts.winsInfix + '.jux-solo-wins')
   
 if opts.winsInfix != None:
  vOutWins = True
 else:
  vOutWins = False	 
   
 mtscc.writeJuxtOut(vlJuxtaposition, vlSysNames, vFullOutFileName, 
                    opts.outScoreTieSplit, vScoreTieFileName, vScoreDifFileName,
                    opts.outRealTieSplit, vRealTieFileName, vRealDifFileName, opts.scoreSortDir, 
                    vOutWins, vlAnyWinFileNames, vlSoloWinFileNames, opts.tabularize, True) 
  

 
##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())


