#! /usr/bin/python

"""	
	This scripts merges n-best lists of multiple MT systems into a distinct
	set of translations (the union of n-bests). It then outputs required
	information for confidence estimation feature eactraction, which will
	ultimately be used in system combination.
	
	Version 1.0
	
"""

import sys, optparse, mtnbest

##======================================================================
## main
def main(argv=None):
	if argv is None:
		argv = sys.argv
	
	parser = optparse.OptionParser(usage="%prog <SYSTEM NAMES> <N-BEST NAMES> <SOURCE SET> [options]" + 
										 "\nMerges n-best lists of multiple MT systems into a distinct and outputs required information for confidence estimation feature eactraction set of translations.", version="%prog 1.0")
	
	parser.add_option("--out-file-name", help="prefix for generated output files [default: merged.nbest.ce]", dest="outFilename", default="merged.nbest.ce", action="store")
	
	(opts, posArgs) = parser.parse_args()
	
	if len(posArgs) < 3:
		parser.error("At least 3 argument is required (comma-separated list of system names, comma-separated list of system n-best absolute files, source absolute file")
	
	# processing arguments
	 
	vlSysNames = posArgs[0].split(',')
	vSysNum = len(vlSysNames)
	vlNBestFilenames = posArgs[1].split(',')
	vSourceFilename = posArgs[2]
	
	# openning source set
	
	try:
		vfSourceSet = open(vSourceFilename)
	except IOError:
		sys.exit("Cannot open source data set file: " + vSourceFilename)
	
	# creating output file 
	
	try:
		vfOutFile = open(opts.outFilename, 'w')
	except IOError:
		sys.exit("Cannot create output file: " + opts.outFilename)
	
	
	# loading n-best lists for all systems
	vlNBestLists = []
	vSysCntr = 0
	for vSysNBestFileName in vlNBestFilenames:
		print "Loading n-best of " + vlSysNames[vSysCntr] + "..."
		vlNBestLists.append(mtnbest.extractSentenceNBests(vSysNBestFileName.strip(), False))
		vSysCntr += 1
		
	
	## main loop
	## iterating through source sentences to extarct and merge n-bests
	vSentenceNo = 0
	for vSrcSent in vfSourceSet:
		print "Processing sentence " + str(vSentenceNo + 1) + " ..."
		
		## merging n-bests of systems for the sentence into a distinct 
		## set of translation (union of n-bests)
		vlMergedNBest = []
		vlMergedNBest = mtnbest.mergeNBests([NBest[vSentenceNo] for NBest in vlNBestLists])
		
		## writing output
		## sentence no ||| source sentence ||| translation ||| space-separated scores ||| space-separated ranks ||| agreement degree
		for vNBestItem in vlMergedNBest:
			vfOutFile.write(str(vSentenceNo + 1) + " ||| ")
			vfOutFile.write(vSrcSent.strip() + " ||| ")
			vfOutFile.write(vNBestItem.trans + " ||| ")
			vfOutFile.write(' '.join([str(score) for score in vNBestItem.sysscores]) + " ||| ")
			vfOutFile.write(' '.join([str(item) for item in vNBestItem.memidxs]) + " ||| ")
			vfOutFile.write(str(vNBestItem.degree))
			vfOutFile.write('\n')
			
		
		vSentenceNo += 1
		
	
	vfOutFile.close()
	

##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())


