#! /usr/bin/python

"""	
	This module defines functions to machine translate text using various
	available tools and models.
	
	Version: 1.5											(26-Jun-2014)
	- Score threshold are added to loadMosesLTTable() of LTTable.
	- Score threshold are added to loadMosesPTTable() of PTTable.
	- writeMosesPTTable() is added to PTTable.
	
	Version: 1.4											(10-Jun-2014)
	- PhraseTable is renamed to PTTable.
	- LTTable is added.
	
	Version: 1.3											(13-May-2014)
	- PhraseTable is added.
	
	Version: 1.2											(24-Mar-2014)
	- findSrcSentence(), _findSrcSentAll(), _findSrcSentFirst(), getTrgAt(),
	  getTrgOf(), tokenizeSrc(), tokenizeTrg() are added to ParallelCorpora.
	- ParallelCorpora constructor is updated to optionally tokezine the
	  corpora wile loading.
	 
	Version: 1.1											(24-Feb-2014)
	- MT class is added.
	- transACCEPT() and transBing() are moved under MT class as methods
	  and renamed to _transACCEPT() and _transBing() which are called by
	  new method translate().
	- WordAlignment is added with loadAlignment(), _loadMosesAlignment().
	- ParallelCorpora is added with size(), _loadCorpus(), loadAlignments(),
	  _loadMosesAlignments().
	- getTrgAlignedTo(), getSrcAlignedTo(), getTrgPositions(), getSrcPositions(), 
	  subsetBySrcPos(), addAlignment(), subsetManyToOne(), isEmpty() are 
	  added to WordAlignment.
	
	Version: 1.0											(22-Feb-2013)
	- transACCEPT()
	- transBing() 
	
"""


import sys, urllib, msmt, util, nlp


class MT:
	'''
	Class for machine translation
	
	TODO: preliminary version; extend
	'''
	
	def __init__(self, pSrcLang, pTrgLang, pSystem):
		'''
		Constructor
		'''
		
		self.srcLang = util.normalizeLangID(pSrcLang)
		self.trgLang = util.normalizeLangID(pTrgLang)
		
		# translator
		self.sys = pSystem.lower()
		# normalizing translator
		if self.sys.startswith("accept"):
			self.sys = "accept"
		elif self.sys.startswith("bing") or self.sys == "ms" or self.sys.startswith("microsoft"):
			self.sys == "bing"
		elif self.sys.startswith("google"):
			self.sys == "google"
		
	
	
	def translate(self, pInput, pSysParam):
		'''
		Translates the input
		
		pSysParam is the system-specific parameter.
		TODO: improve system parameter passing
		'''
		
		if self.sys == "accept":
			self._transACCEPT(pInput, pSysParam)
		elif self.sys == "bing":
			self._transBing(pInput, pSysParam)
		
	
	
	def _transACCEPT(pInput, pSystem):
		'''
		Translates using ACCEPT online translator.
		
		pSystem: specifies the type of system to be use: symantec (trained on 
		TM and supplements) TWB (trained on TWB data and supplements)
		
		Find documentation in http://www.accept.unige.ch/Products/D_4_1_Baseline_MT_systems.pdf
		'''
		
		import json
		
		cURL = "http://accept:motelone@accept.statmt.org/demo/translate.php"
		
		if pSystem.lower().startswith("s"):
			vSystem = "sb"
		elif pSystem.lower().startswith("t"):
			vSystem = "tb"
		
		vlOutput = []
		for vInput in pInput.strip().split('\n'):
			vParams = urllib.urlencode({'v' : '1.0', \
			                            'ie' : 'UTF8', \
			                            'langpair' : '%s|%s' % (self.srcLang, self.trgLang), \
			                            'system' : vSystem, \
			                            'q' : vInput}) 
			vfOut = urllib.urlopen(cURL, vParams) 
			vOutput = vfOut.readline()
			vResponse = json.loads(vOutput) 
			if not vResponse['responseData']: 
				sys.exit("Error: %s" % vResponse['responseDetails'])
			else:
				vlOutput.append(vResponse['responseData']['translatedText'])
		
		return '\n'.join(vlOutput)
		
	
	
	def _transBing(pInput, pCredential = 0):
		'''
		NOTE: ADD OPTION TO TRANSLATE TEXT INSTEAD OF SEGMENTS WITH E.G. 5000 
		      CHARACTERS LIMIT.
		
		Translates using bing (Microsoft translator).
		'''
		
		# first credentials from Raphael's registration
		vlBingClientIDs = ["msmt_dcu", "c41195af-1908-4681-a362-f5481aeb9f25", "5f5a4666-7bda-4515-894a-594643163435"]
		vlBingClientKeys = ["+QYp9bf5/FTMBM6CpQfCg7FxYbaYy7bnxS0pYbXYD/g=", "H/LJ7rg3qItAcuwokJgBQD9mK3AxExojApKCLBf+7bk=", "YAxKYfmuVNU0mDsBwzJaO2aCgZz1BLOPznm4NVQt0Qc="]
		
		vBingClientID = vlBingClientIDs[pCredential]
		vBingClientKey = vlBingClientKeys[pCredential]
		vAccToken = msmt.get_access_token(vBingClientID, vBingClientKey)
		
		vlOutput = []
		vlEmptyOutput = []
		for i, vInput in enumerate(pInput.strip().split('\n'), start = 1):
			vResponse = ""
			print i
			while True:
				vResponse = msmt.translate(vAccToken, vInput, self.trgLang, self.srcLang)
				
				# renewing the key if the server didn't respond
				if vResponse == None:
					vAccToken = msmt.get_access_token(vBingClientID, vBingClientKey)
					continue
				else:
					break
			
			if vResponse.strip() == "":
				vlEmptyOutput.append(i)
				vlOutput.append("")
			else:
				# stripping XML
				vOutput = vResponse[vResponse.find('>') + 1 : vResponse.rfind('<')]
				vlOutput.append(vOutput)
		
		if len(vlEmptyOutput) > 0:
			print "The following lines had empty translation output: %s" % ','.join(vlEmptyOutput)
		
		return '\n'.join(vlOutput)
		
	
	


class WordAlignment:
	'''
	Class for word (token) alignment of a sentence pair
	
	Word and token are used interchangeably.
	
	Preliminary; extend
	'''
	
	def __init__(self):
		'''
		Constructor
		'''
		
		## word-alignment for the sentence pair (list of word-to-word alignments) 
		## See addAlignment() for details of the format alignments are stored.
		## If the storage method changes, the following methods should be
		## updated:
		## - __init__()
		## - addAlignment()
		## - isEmpty()
		## - getSrcPositions()
		## - getTrgPositions()
		## - getSrcAlignedTo()
		## - getTrgAlignedTo()
		self._alignment = []
		
	
	
	def addAlignment(self, plSrcPositions, plTrgPositions):
		'''
		Adds new alignments from all given source positions into given 
		target positions
		'''
		
		## Alignment is stored in a list of tuples of source and target 
		## token positions (e.g. [(1, 3), (2, 1), (3, 2)]).
		## Order is not important though it may be already sorted by the
		## aligner.
		
		for vSrcPos in plSrcPositions:
			for vTrgPos in plTrgPositions:
				vNewAlignment = (vSrcPos, vTrgPos)
				if vNewAlignment not in self._alignment:
					self._alignment.append(vNewAlignment)
		 
	
	
	def isEmpty(self):
		'''
		Returns true if there is no word alignment in the object
		'''
		
		return len(self._alignment)
		  
	
	
	def loadAlignment(self, pAlignment, pFormat):
		'''
		Loads alignment which is in the specified format
		'''
		
		vFormat = pFormat.lower()
		if vFormat == "moses":
			self._loadMosesAlignment(pAlignment)
		
	
	
	def _loadMosesAlignment(self, pAlignment):
		'''
		Loads Moses alignment for the sentence pair
		
		Moses format is the same format used by this class, but it uses
		zero-based indecies as token IDs. We use token position (one-based).
		'''
		
		## assumed format: 0-0 0-1 1-2 2-3 where numbers are zero-based 
		## word indecies
		
		vlIdxPairs = pAlignment.split(' ')
		
		for vIdxPair in vlIdxPairs:
			vlIdxPair = vIdxPair.split('-')
			self.addAlignment([int(vlIdxPair[0]) + 1], [int(vlIdxPair[1]) + 1])
		
	
	
	def getTrgPositions(self):
		'''
		Returns positions of target side of the alignment
		'''
		
		return list(set([a[1] for a in self._alignment]))
		
	
	
	def getSrcPositions(self):
		'''
		Returns positions of target side of the alignment
		'''
		
		return list(set([a[0] for a in self._alignment]))
		
	
	
	def getTrgAlignedTo(self, pSrcPos):
		'''
		Returns positions of target words aligned to source word at given 
		position 
		'''
		
		return [a[1] for a in self._alignment if a[0] == pSrcPos]
		
	
	
	def getSrcAlignedTo(self, pTrgPos):
		'''
		Returns positions of source words aligned to target word at given 
		position 
		'''
		
		return [a[0] for a in self._alignment if a[1] == pTrgPos]
		
	
	
	def subsetBySrcPos(self, plSrcPositions):
		'''
		Creates and returns a WordAlignment object which is a subset of 
		this alignment with only items whose source posistions are in the
		given list
		'''
		
		vSSWA = WordAlignment()
		
		for vSrcPos in plSrcPositions:
			vSSWA.addAlignment([vSrcPos], self.getTrgAlignedTo(vSrcPos))
		
		return vSSWA
		
	
	
	def subsetManyToOne(self):
		'''
		Creates and returns a WordAlignment object which is a subset of 
		this alignment with each target word is aligned to multiple source
		words
		'''
		
		vSSWA = WordAlignment()
		
		for vTrgPos in self.getTrgPositions():
			vlAlignedSrcPositions = self.getSrcAlignedTo(vTrgPos)
			if len(vlAlignedSrcPositions) > 1:
				vSSWA.addAlignment(vlAlignedSrcPositions, [vTrgPos])
		
		return vSSWA
		
	
	


class ParallelCorpora:
	'''
	Class for sentence-aligned parallel corpora
	'''
	
	def __init__(self, pSrcCorpusFilename, pSrcLanguage, pTrgCorpusFilename, pTrgLanguage, pflgTokenize = False):
		'''
		Constructor
		'''
		
		self.srcCorpus = self._loadCorpus(pSrcCorpusFilename, pSrcLanguage, pflgTokenize)
		self.srcLanguage = pSrcLanguage
		self.trgCorpus = self._loadCorpus(pTrgCorpusFilename, pTrgLanguage, pflgTokenize)
		self.trgLanguage = pTrgLanguage
		
		# sanity check
		vSrcSize = len(self.srcCorpus)
		vTrgSize = len(self.trgCorpus)
		if vSrcSize != vTrgSize:
			raise Exception("Source and target corpus sizes do not match: %s vs. %s" % (vSrcSize, vTrgSize))
		
		## list of alignments (of type WordAlignment) one element for each
		## sentence pair
		self.alignments = []
		
	
	
	@property
	def size(self):
		'''
		Returns the size of parallel corpora
		'''
		
		return len(self.srcCorpus)
		
	
	
	def _loadCorpus(self, pCorpusFilename, pLanguage, pflgTokenize = False):
		'''
		Loads corpus from file into list of lines and optionally tokenizes
		them
		'''
		
		vlLines =  open(pCorpusFilename).read().strip().split('\n')
		
		if pflgTokenize:
			return [nlp.tokenizeSegment(l, pLanguage, pflgPrint = False) for l in vlLines]
		else:
			return vlLines
		
	
	
	def tokenizeSrc(self, pflgGauge):
		'''
		Tokenizes source side
		'''
		
		self.srcCorpus = self._tokenize(self.srcCorpus, self.srcLanguage, pflgGauge)
		
	
	
	def tokenizeTrg(self, pflgGauge):
		'''
		Tokenizes target side
		'''
		
		self.trgCorpus = self._tokenize(self.trgCorpus, self.trgLanguage, pflgGauge)
		
	
	
	def _tokenize(self, plLines, pLanguage, pflgGauge):
		'''
		Tokenizes given lines
		'''
		
		vlTokenized = []
		for i, vSent in enumerate(plLines):
			vlTokenized.append(nlp.tokenizeSegment(vSent, pLanguage, pflgPrint = False))
			
			if (i + 1) % 100000 == 0:
				sys.stdout.write('!')
			elif (i + 1) % 1000 == 0:
				sys.stdout.write('.')
				sys.stdout.flush()
		
		return vlTokenized
		
	
	
	def loadAlignments(self, pAlignmentFilename, pFormat):
		'''
		Loads alignments of the parallel corpora
		'''
		
		vFormat = pFormat.lower()
		if vFormat == "moses":
			self._loadMosesAlignments(pAlignmentFilename)
		
	
	
	def _loadMosesAlignments(self, pAlignmentFilename):
		'''
		Loads Moses alignments for the corpora from file
		'''
		
		vlRawAlignments = open(pAlignmentFilename).read().strip().split('\n')
		
		# sanity check
		vAlignmentSize = len(vlRawAlignments)
		if self.size != vAlignmentSize:
			raise Exception("Corpus size does not match alignment size: %s vs. %s" % (self.size, vAlignmentSize))
		
		for vSrcSent, vTrgSent, vRA in zip(self.srcCorpus, self.trgCorpus, vlRawAlignments):
			vWA = WordAlignment(vSrcSent, vTrgSent)
			vWA.loadAlignment(vRA, pFormat)
			self.alignments.append(vWA)
			
		
	
	
	def findSrcSentence(self, pSentence, pflgFindAll = True):
		'''
		Looks for the given sentence in the source side of the parallel
		corpora and returns its location if found.
		
		The location starts from 1.
		
		According to pflgFindAll, it finds all occurrences or only the 
		first one.
		'''
		
		if pflgFindAll:
			return self._findSrcSentAll(pSentence)
		else:
			return self._findSrcSentFirst(pSentence)
		
	
	
	def _findSrcSentAll(self, pSentence):
		'''
		Looks for the given sentence in the source side of the parallel
		corpora and returns the locations of all occurrences. 
		
		An empty list will be returned if nothing is found.
		
		The location starts from 1.
		'''
		
		vlFound = []
		
		for i, vSrcSent in enumerate(self.srcCorpus):
			if pSentence == vSrcSent:
				vlFound.append(i + 1)
		
		return vlFound
		
	
	
	def _findSrcSentFirst(self, pSentence):
		'''
		Looks for the given sentence in the source side of the parallel
		corpora and returns the locations of its first occurrence if found.
		
		-1 will be returned if nothing is found.
		
		The location starts from 1.
		'''
		
		for i, vSrcSent in enumerate(self.srcCorpus):
			if pSentence == vSrcSent:
				return i + 1
		
		return -1
		
	
	
	def getTrgAt(self, pPosition):
		'''
		Returns the target sentence at a given position
		
		Position starts from 1.
		'''
		
		return self.trgCorpus[pPosition - 1]
		
	
	
	def getTrgOf(self, pSrcSentence):
		'''
		Returns the target pair of a given source sentence if the source
		found
		
		If multiple instances of the source exists, it returns one in no
		special order.
		'''
		
		vSrcPos = self.findSrcSentence(pSrcSentence, pflgFindAll = False)
		
		if vSrcPos == -1:
			raise LookupError("Source sentence not found!")
		else:
			return self.getTrgAt(vSrcPos)
		
	
	


class PTTable:
	'''
	Class for phrase translation table
	'''
	
	def __init__(self):
		'''
		Constructor
		'''
		
		## Translation table is stored in dictionary which internally uses 
		## hash table and is fast. The keys are the source phrases. The 
		## values are list of translations. Each translation is represented 
		## via a tuple the first element of which is the translation and 
		## the second is a dictionary of following scores: 
		## - direct translation probability
		## - direct lexical weighting
		## - inverse translation probability
		## - inverse lexical weighting
		self.table = {}
		
	
	
	def loadMosesPTTable(self, pPTFilename, pAvgScoreThreshold = 0, pAvgPTPThreshold = 0, pDPTPThreshold = 0, pIPTPThreshold = 0, pflgGauge = False):
		'''
		Loads phrase table from Moses format
		
		An score threshold can be set to load only records meeting the threshold:
		- pAvgScoreThreshold: threshold on average of all 4 scores
		- pAvgPTPThreshold: threshold on average of 2 phrase translation
		                    probabilities
		- pDPTPThreshold: threshold on direct phrase translation probability
		- pIPTPThreshold: threshold on inverse phrase translation probability
		
		Though all thresholds will be checked to meet, setting only one of
		the would be preferable.
		'''
		
		import gzip
		
		if pPTFilename.endswith(".gz"):
			vPTFile = gzip.open(pPTFilename)
		else:
			vPTFile = open(pPTFilename)
		
		for i, vPTLine in enumerate(vPTFile):
			if pflgGauge:
				if (i + 1) % 100000 == 0:
					sys.stdout.write('!')
					sys.stdout.flush()
				elif (i + 1) % 1000 == 0:
					sys.stdout.write('.')
					sys.stdout.flush()
			
			vPTLineSplit = vPTLine.split(" ||| ")
			
			vScoreSplit = vPTLineSplit[2].split(' ')
			# older version of Moses generate a fifth score of phrase penalty
			if len(vScoreSplit) == 4:
				vdScores = {"inv-trans-prob": vScoreSplit[0], "inv-lex-weight": vScoreSplit[1], "dir-trans-prob": vScoreSplit[2], "dir-lex-weight": vScoreSplit[3]}
			elif len(vScoreSplit) == 5:
				vdScores = {"inv-trans-prob": vScoreSplit[0], "inv-lex-weight": vScoreSplit[1], "dir-trans-prob": vScoreSplit[2], "dir-lex-weight": vScoreSplit[3], "phrase-penalty": vScoreSplit[4]}
			
			# checking thresholds
			if (pAvgScoreThreshold > (float(vdScores["dir-trans-prob"]) + float(vdScores["inv-trans-prob"]) + float(vdScores["dir-lex-weight"]) + float(vdScores["inv-lex-weight"])) / 4.0 or
			    pAvgPTPThreshold   > (float(vdScores["dir-trans-prob"]) + float(vdScores["inv-trans-prob"])) / 2.0 or
			    pDPTPThreshold     > float(vdScores["dir-trans-prob"]) or
			    pIPTPThreshold     > float(vdScores["inv-trans-prob"])): 
				continue
			
			if vPTLineSplit[0] not in self.table:
				self.table[vPTLineSplit[0]] = [(vPTLineSplit[1], vdScores)]
			else:
				self.table[vPTLineSplit[0]].append((vPTLineSplit[1], vdScores))
		
	
	
	def writeMosesPTTable(self, pPTFilename):
		'''
		Writes the phrase translation table into a file in the new Moses 
		format
		
		The new moses format is:
		Phrase ||| Translation ||| inverse phrase translation probability ||| inverse lexical weighting ||| direct phrase translation probability ||| direct lexical weighting
		
		The phrase are sorted.
		'''
		
		vLines = []
		for vPhrase in sorted(self.table.iterkeys()):
			for vTransRec in self.table[vPhrase]:
				vScores = "%s %s %s %s" % (vTransRec[1]["dir-trans-prob"], vTransRec[1]["inv-trans-prob"], vTransRec[1]["dir-lex-weight"], vTransRec[1]["inv-lex-weight"])
				vLines.append("%s ||| %s ||| %s" % (vPhrase, vTransRec[0], vScores))
		
		open(pPTFilename, 'w').write('\n'.join(vLines) + '\n')
		
	
	
	def getPTransRec(self, pPhrase):
		'''
		Searches for the given source phrase and returns its record which
		is the translation-score tuple if found, empty list otherwise
		'''
		
		try:
			return self.table[pPhrase]
		except KeyError:
			return []
		
	
	
	def getPTrans(self, pPhrase):
		'''
		Searches for the given source phrase and returns its translations 
		if found, empty list otherwise
		'''
		
		try:
			return [trans[0] for trans in self.table[pPhrase]]
		except KeyError:
			return []
		
	
	
	def getPTransScores(self, pPhrase):
		'''
		Searches for the give source phrase and returns its translation 
		scores in a dictionary if found, empty list otherwise
		'''
		
		try:
			return [trans[1] for trans in self.table[pPhrase]]
		except KeyError:
			return []
		
	
	


class LTTable:
	'''
	Class for lexical translation table
	'''
	
	def __init__(self):
		'''
		Constructor
		'''
		
		## Translation table is stored in dictionary which internally uses 
		## hash table and is fast. The keys are the source words. The 
		## values are list of translations. Each translation is represented 
		## via a tuple the first element of which is the translation and 
		## the second is the translation scores (maximum likelihood).
		self.table = {}
		
	
	
	def loadMosesLTTable(self, pLTFilename, pScoreThreshold = 0):
		'''
		Loads lexical translation table from Moses format (training step 4)
		'''
		
		for vTTLine in open(pLTFilename):
			vTTLineSplit = vTTLine.strip().split(" ")
			
			vSrcWord = vTTLineSplit[0]
			vTrgWord = vTTLineSplit[1]
			vScore = vTTLineSplit[2]
			
			if float(vScore) >= pScoreThreshold:
				if vSrcWord not in self.table:
					self.table[vSrcWord] = [(vTrgWord, vScore)]
				else:
					self.table[vSrcWord].append((vTrgWord, vScore))
		
	
	
	def getLTransRec(self, pWord):
		'''
		Searches for the given source word and returns its record which
		is the translation-score tuple if found, empty list otherwise
		'''
		
		try:
			return self.table[pWord]
		except KeyError:
			return []
		
	
	
	def getLTrans(self, pWord, pflgSort = True):
		'''
		Searches for the given source word and returns its translations 
		if found, empty list otherwise
		
		The translations can be sorted in descending order in terms of 
		their score.
		'''
		
		try:
			if pflgSort:
				return [trans[0] for trans in sorted(self.table[pWord], key = lambda x: -float(x[1]))]
			else:
				return [trans[0] for trans in self.table[pWord]]
		except KeyError:
			return []
		
	
	
	def getLTransScores(self, pWord):
		'''
		Searches for the give source word and returns its translation 
		scores if found, empty list otherwise
		'''
		
		try:
			return [trans[1] for trans in self.table[pWord]]
		except KeyError:
			return []
		
	

