#! /usr/bin/python

"""	This script scores a set of translations by various metrics using 
	external tools.
	
	When available and requested, it also caculates and outputs statistical
	significance.
	
	The input is the translation file, reference file(s). In case of 
	statistical significance test, two translations should be provided.
	For each selected metric, its required parameters should be provided.
	
	The output is the scores reports is various formats specific to the 
	metrics used.
	
	For some options, like the paths to scorers, it first looks for the
	config file (by default the score-translation.ini file in the same
	path as script), then overrides them command options if they exist.
	
	NOTE: NEW SCORING FUNCTIONS ARE STARTED TO BE WRITTEN IN mt.py. THIS
	      SCRIPT IS NO MORE CONTINUED AND SHOULD BE TRANSFERRED TO mt.py.
	
	Version: 1.2											(12-Mar-2013)
	- Naming TER score files changed to include ".ter".
	
	Version: 1.1											(02-Aug-2012)
	
	- <SYSTEM NAME> <DATA SET ID> <SOURCE LANGUAGE> <TARGET LANGUAGE> are
	  changed to options to ease the use of the scripts
	- Using configuartion file is added
	- Explicit options for requesting any type of scoring
	
	Version: 1.0
	- BLEU and NIST and their statistical significant using mteval v13a
	  and CMU ARK tools is implemented.
	- TER scoring is implemented (tercom: Snover et al. (2006))
"""  

from ConfigParser import SafeConfigParser
import sys, optparse, mteval, mtnbest, os


##======================================================================
## main
def main(argv=None):
	if argv is None:
		argv = sys.argv
 
	parser = optparse.OptionParser(usage="%prog <TRANSLATION FILE> <REFRENCE FILE> [options]" + 
										 "\nThis script scores a set of translations by various metrics using external tools.", version="%prog 1.0")

	parser.add_option("--config", help="configuration (ini) file", dest="conf", default="score-translation.ini", action="store")
	parser.add_option("--sys-name", help="system name", dest="sysName", default="NewSys", action="store")
	parser.add_option("--dataset-id", help="dataset ID", dest="datasetID", default="Dataset", action="store")
	parser.add_option("--src-lang", help="source language", dest="srcLang", default="Any", action="store")
	parser.add_option("--trg-lang", help="target language", dest="trgLang", default="Any", action="store")
	parser.add_option("--ter", help="score TER", dest="scoreTER", action="store_true")
	parser.add_option("--mteval13a", help="score MTEval v13a", dest="scoreMTEval13a", action="store_true")
	parser.add_option("--ter-scorer-path", help="path to tercom scorer's java class", dest="terScorer", action="store")
	parser.add_option("--ter-out-type", "--tero", help="output type for tercom (leave to generate all types)", dest="terOutType", default="ter", action="store")
	parser.add_option("--mteval13a-scorer-path", help="path to mteval-v13a.pl script", dest="mteval13aPath", action="store")
	parser.add_option("--mteval13a-perllib", help="path to perl library used by MTEval v13a", dest="mteval13aPerlLib", action="store")
	parser.add_option("--mteval13a-test-sig", help="test statistical significance for mreval v13a", dest="testMTEval13aSig", action="store_true")
	parser.add_option("--mteval13a-statsig-path", "--mteval13a-statsig-path", help="path to CMU ARK tool for statistical significance test with MTEval v13a scores", dest="mteval13aStatSigPath", action="store")
	parser.add_option("--statsig-baseline", help="baseline system translation for statistical significance test", dest="baseTrans", action="store")
	parser.add_option("--statsig-base-sysname", help="name of baseline system used for statistical significance test", dest="baseSysID", default="BaseLine", action="store")
	parser.add_option("--source-file", help="source sentences (needed for scorers like MTEval)", dest="sourceFile", action="store")
	parser.add_option("--keep-xml", help="keep all XML files generated for MTEval scoring", dest="keepXMLs", action="store_true")
 
	(opts, posArgs) = parser.parse_args()
 
	if len(posArgs) < 2:
		parser.error("At least 2 arguments are required")
	
	# processing configuartion file
	## Note: common options will be overriden by command options (optparse)
	
	vAppPath = os.path.dirname(__file__)
	vConfigParser = SafeConfigParser()
	vConfigParser.read(os.path.join(vAppPath, opts.conf))
	vTerScorer = vConfigParser.get("TER","terScorer")
	vTerOutType = vConfigParser.get("TER","terOutType")
	vMTEval13aPath = vConfigParser.get("MTEval13a","mteval13aPath")
	vMTEval13aPerlLib = vConfigParser.get("MTEval13a","mteval13aPerlLib")
	vMTEval13aStatSig = vConfigParser.get("MTEval13a","mteval13aStatSigPath")
	
	
	# processing arguments
	 
	vTransFilename = posArgs[0]
	vRefFilename = posArgs[1]
	
	# processing options
	
	vSysID = opts.sysName
	vDatasetID = opts.datasetID
	vSrcLang = opts.srcLang
	vTrgLang = opts.trgLang
	if opts.terScorer != None:
		vTerScorer = opts.terScorer
	if opts.terOutType != None:
		vTerOutType = opts.terOutType
	if opts.mteval13aPath != None:
		vMTEval13aPath = opts.mteval13aPath
	if opts.mteval13aPerlLib != None:
		vMTEval13aPerlLib = opts.mteval13aPerlLib
	if opts.mteval13aStatSigPath != None:
		vMTEval13aStatSig = opts.mteval13aStatSigPath
	
	# scoring with the requested metrics
	
	if opts.scoreTER:
		print "Scoring with TER (tercom tool) ..."
		# openning devnull to redirect the log output of TER scorer
		vDevnull = open(os.devnull, 'w')
		vOutPrefix = vTransFilename + ".ter"
		mteval.scoreTERcom(vTransFilename, vRefFilename, vTerScorer, vOutPrefix, [], vTerOutType, False, vDevnull)
		print "Look for " + vOutPrefix + '.' + opts.terOutType + " for TER score output"
	
	if opts.scoreMTEval13a:
		print "Scoring with MTEval v13a (CMU ARK tool) ..."
		mteval.scoreMTEval13a(vTransFilename, vRefFilename, opts.sourceFile,
							  vMTEval13aPath, "mteval-v13a.pl", vMTEval13aPerlLib,
							  "", 0, [],
							  opts.testMTEval13aSig, vMTEval13aStatSig, "mteval-v13a-sig.pl"
							  vDatasetID, vDatasetID, vSrcLang, vTrgLang, vSysID, vDatasetID, "", opts.keepXMLs,
							  opts.baseTrans, opts.baseSysID)
		print "Look for " + vTransFilename + ".mteval13a for MTEval v13a score output for the hypothesis translation"
		if opts.baseTrans:
			print "Look for " + opts.baseTrans + ".mteval13a for MTEval v13a score output for the baseline translation"
		if opts.testMTEval13aSig:
			print "Look for " + vTransFilename + ".mteval13a-sig for statistical significance test results"

 
##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())

