#! /usr/bin/python

"""	
This script wraps the input file content in XML complying with NIST 
mteval v13a format.

Note that all XML special charcters in the content are masked.
"""

import sys, optparse, mteval

##======================================================================
## main
def main(argv=None):
	if argv is None:
		argv = sys.argv
 
	parser = optparse.OptionParser(usage="%prog <INPUT FILE> <DATA SET TYPE: srcset|refset|tstset> <DATA SET ID> <DOCUMENT ID> [options]" + 
										 "\nWraps the input file content in NIST mteval v13a XML format.", version="%prog 1.0")

	parser.add_option("--srclang", help="source language (for all data sets)", dest="srcLang", action="store")
	parser.add_option("--trglang", help="target language (for translation (test and reference) data sets)", dest="trgLang", action="store")
	parser.add_option("--sysid", help="system ID (for test sets)", dest="sysID", action="store")
	parser.add_option("--refid", help="reference ID (for reference sets)", dest="refID", action="store")
 	parser.add_option("--genre", help="data genre (for all data sets)", dest="genre", default="default", action="store")
 
	(opts, posArgs) = parser.parse_args()
 
	# procesing arguments
	# NOTE: At the moment, no control is taken for option combination
	
	if len(posArgs) < 4:
		parser.error("At least 4 argument is required!")
		
	vInputFileName = posArgs[0]
	vDatasetType = posArgs[1]
	vDatasetID = posArgs[2]
	vDocID = posArgs[3]
	
	
	# wrapping and outputing
	mteval.wrapMTEvalXML(vInputFileName, vDatasetType, vDatasetID, vDocID, opts.srcLang, opts.trgLang, opts.sysID, opts.refID, opts.genre, sys.stdout)
	
##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())


