#! /usr/bin/python
# -*- coding: utf-8 -*-

## This script masks specific patterns in the input string. 
##
## Current version: 1.0
##


import sys, optparse, re


##======================================================================
## main
def main(argv=None):
	if argv is None:
		arv = sys.argv
	
	parser = optparse.OptionParser(usage="%prog [OPTIONS]" + 
										 "\nMasks specific patterns in the input sentence.", version="%prog 1.0")

	parser.add_option("-a", "--all", help="mask every implemented pattern", dest="maskAll", action="store_true")
	parser.add_option("-c", "--htmlspecial", help="mask HTML special character(e.g. &amp)", dest="maskHTMLSpecial", action="store_true")
	parser.add_option("-s", "--specialstr", help="mask special strings (e.g. C4:8A:1D:)", dest="maskSpecialStr", action="store_true")
	parser.add_option("-r", "--regkey", help="mask windows registry key", dest="maskRegKey", action="store_true")
	parser.add_option("-p", "--pholder", help="mask placeholders", dest="maskPHolder", action="store_true")
	parser.add_option("-l", "--listnum", help="mask list numbers", dest="maskListNum", action="store_true")
	parser.add_option("-m", "--menuitem", help="mask menu item", dest="maskMenuItem", action="store_true")
	parser.add_option("-t", "--tag", help="mask HTML/XML tags", dest="maskTag", action="store_true")
	parser.add_option("-v", "--var", help="mask variables", dest="maskVar", action="store_true")
	parser.add_option("-b", "--bslash", help="replace \n \r \t with white space", dest="maskBSlash", action="store_true")
	parser.add_option("-e", "--email", help="mask Email", dest="maskEmail", action="store_true")
	parser.add_option("-u", "--url", help="mask URL", dest="maskURL", action="store_true")
	parser.add_option("-w", "--winpath", help="mask Windows file path", dest="maskWPath", action="store_true")
	parser.add_option("-x", "--unixpath", help="mask Unix file path", dest="maskUPath", action="store_true")
	parser.add_option("-i", "--ipverno", help="mask IP address and version number", dest="maskIPV", action="store_true")
	parser.add_option("-d", "--datetime", help="mask date/time", dest="maskDateTime", action="store_true")
	### add pattern options here
	
	(opts, posArgs) = parser.parse_args()

	# processing options

	## overriding individual pattern options if --all is set 
	### remember to add new feature options here
	if opts.maskAll:
		opts.maskHTMLSpecial = True
		opts.maskSpecialStr = True
		opts.maskRegKey = True
		opts.maskPHolder = True
		opts.maskListNum = True
		opts.maskMenuItem = True
		opts.maskTag = True
		opts.maskVar = True
		opts.maskBSlash = True
		opts.maskEmail = True
		opts.maskURL = True
		opts.maskWPath = True
		opts.maskUPath = True
		opts.maskIPV = True
		opts.maskDateTime = True
	
	
	# variables to keep track of the number of replacement for each pattern.
	vTotalHSpecials = 0
	vHSpecialLines = 0
	vTotalSStrs = 0
	vSStrLines = 0
	vTotalRegKeys = 0
	vRegKeyLines = 0
	vTotalPHolders = 0
	vPHolderLines = 0
	vTotalLNums = 0
	vLNumLines = 0
	vTotalMItems = 0
	vMItemLines = 0
	vTotalTags = 0
	vTagLines = 0
	vTotalVars = 0
	vVarLines = 0
	vTotalBSlashs = 0
	vBSlashLines = 0
	vTotalEmails = 0
	vEmailLines = 0
	vTotalURLs = 0
	vURLLines = 0
	vTotalWPaths = 0
	vWPathLines = 0
	vTotalUPaths = 0
	vUPathLines = 0
	vTotalIPVs = 0
	vIPVLines = 0
	vTotalDTs = 0
	vDTLines = 0
	
	# reading input string from stdin and processing
	
	## Note: in cases where the mask is surrounded by a space, the intention
	## is to treat fusion alongside. E.g. follow:http://. -> follow -URL- .
	## Since this adds an unwanted space for non-fused cases, a tokenization
	## or cleaning of trailing spaces is advised.
	
	vLineCntr = 0
	while True:
		vInput = sys.stdin.readline()
		if not vInput.strip():
			break

		# masking HTML special characters
		if opts.maskHTMLSpecial:
			vdHTMSpec = {'&amp;trade;':'', '&amp;':'&', '&apos;':"'", '&copy;':'(C)', '&dagger;':'', '&eacute;':'', '&euml;':'', '&euro;':'', '&hellip;':'...', \
			             '&ldquo;':'``', '&rdquo;':"''", '&lsquo;':'`', '&rsqu?o;':"'", '&[mn]d?ash;':'-', '(?:&nbsp;|&nbps;|&space;)+':' ', '&oslash;':'', '&quot;':'"', \
						 '&laquo;':'', '&raquo;':"", '&reg;':'', '&trade;':'', '&uuml;':''}
			## Since the number of occurrences is needed, we don't use
			## replace function.
			vHSpecials = 0
			for p, m in vdHTMSpec.iteritems():
				vPattern = re.compile(p, re.IGNORECASE)
				vInput, vNum1 = vPattern.subn(m, vInput)
				vHSpecials += vNum1
			vTotalHSpecials += vHSpecials
			if vHSpecials > 0:
				vHSpecialLines += 1
				vNum1 = 0

		# masking special strings seen in data
		if opts.maskSpecialStr:
			# e.g. 60:06:01:60:83:B0:11:00:4A:C4:8A:1D:
			vPattern = re.compile(r"(?:(?<=[\s#])|^)(?:[\d\w]{2,4}:){4,}[\d\w]*")
			vInput, vNum1 = vPattern.subn(r" -SPS-", vInput)
			vTotalSStrs += vNum1
			if vNum1 > 0:
				vSStrLines += 1
				vNum1 = 0

		# masking windows registry keys
		if opts.maskRegKey:
			#vPattern = re.compile(r"(^|[ #(\[])HKEY_.*?([ #).\n\]])")
			vPattern = re.compile(r"HKEY_.*?([\s#)\"',\].])")
			vInput, vNum1 = vPattern.subn(r" -WRK- \1", vInput)
			vTotalRegKeys += vNum1
			if vNum1 > 0:
				vRegKeyLines += 1
				vNum1 = 0

		# masking placeholders: \{...\}
		# Note: mask after windows registry keys, before windows file path
		if opts.maskPHolder:
			vPattern = re.compile(r"\\\{.*?\\\}")
			vInput, vNum1 = vPattern.subn(r" -PHL- ", vInput)
			vTotalPHolders += vNum1
			if vNum1 > 0:
				vPHolderLines += 1
				vNum1 = 0

		# masking list numbers
		# Note: mask before angle-bracket-related patterns
		if opts.maskListNum:
			# N., N-, N) 
			vPattern = re.compile(r"^\s*\d+\s*[\).-](?=\s+[^\d])")
			vInput, vNum1 = vPattern.subn(r"-LNM- ", vInput)
			vTotalLNums += vNum1
			# <N> 
			vPattern = re.compile(r"^\s*<\s*\d+\s*>")
			vInput, vNum2 = vPattern.subn(r"-LNM- ", vInput)
			vTotalLNums += vNum2
			if vNum1 + vNum2 > 0:
				vLNumLines += 1
				vNum1 = 0
				vNum2 = 0

		# masking menu items
		# Note: mask before variable
		## For example, in Click  < >Reports < >, the menu item will be
		## replaced with "Reports". If needed, the quotation marks will 
		## be mask later. In fact, this is not masking but editing.
		if opts.maskMenuItem:
			vPattern = re.compile(r"<\s*>\s*([^<>]*?)\s*<\s*>")
			vInput, vNum1 = vPattern.subn(r'"\1"', vInput)
			vTotalMItems += vNum1
			if vNum1 > 0:
				vMItemLines += 1
				vNum1 = 0

		# masking HTML/XML tags
		# Note: masks the whole tag
		# Note: mask after menu items above and before variable
		if opts.maskTag:
			# HTML-like
			vPattern = re.compile(r"<\w+\s*\w+\=.*?[>/].*>")
			vInput, vNum1 = vPattern.subn(r" -TAG- ", vInput)
			vTotalTags += vNum1
			# XML-like
			vPattern = re.compile(r"<\w+\:.*?[>/].*>")
			vInput, vNum2 = vPattern.subn(r" -TAG- ", vInput)
			vTotalTags += vNum2
			if vNum1 + vNum2 > 0:
				vTagLines += 1
				vNum1 = 0
				vNum2 = 0
		
		# masking variables
		# Note: mask after menu items above
		if opts.maskVar:
			# e.g. <token >AssignOrgGroupTask < >
			vPattern = re.compile(r"<[\w-]+\s*>\s*[^<>]*?\s*<\s*>")
			vInput, vNum1 = vPattern.subn(r" -VAR- ", vInput)
			vTotalVars += vNum1
			# fusions like [..]X...
			# Note: the masked pattern could even be removed!
			vPattern = re.compile(r"(?:\s|^)\[[^\]]*?\]([A-Z])")
			vInput, vNum2 = vPattern.subn(r"-VAR- \1", vInput)
			vTotalVars += vNum2
			# [..], <..> 
			vPattern = re.compile(r"[<\[].*?[>\]]")
			vInput, vNum3 = vPattern.subn(r"-VAR-", vInput)
			vTotalVars += vNum3
			if vNum1 + vNum2 + vNum3 > 0:
				vVarLines += 1
				vNum1 = 0
				vNum2 = 0
				vNum3 = 0

		# replacing \n \r \t with whitespace
		# Note: mask before windows file path
		if opts.maskBSlash:
			vPattern = re.compile(r"(?:\s+(?:\\+[ntr])+)+")
			vInput, vNum1 = vPattern.subn(r" ", vInput)
			vTotalBSlashs += vNum1
			vPattern = re.compile(r"(?:([?.,\[\}]+)(?:\\+[ntr])+)+")
			vInput, vNum2 = vPattern.subn(r"\1 ", vInput)
			vTotalBSlashs += vNum2
			vPattern = re.compile(r"(?:\\+[ntr])+$")
			vInput, vNum3 = vPattern.subn(r"", vInput)
			vTotalBSlashs += vNum3
			# this should be done last
			vPattern = re.compile(r"(?:\\+[ntr]){2,}")
			vInput, vNum4 = vPattern.subn(r" ", vInput)
			vTotalBSlashs += vNum4
			if vNum1 + vNum2 + vNum3 + vNum4 > 0:
				vBSlashLines += 1
				vNum1 = 0
				vNum2 = 0
				vNum3 = 0
				vNum4 = 0

		# masking Email
		# Note: Email should be masked before URL
		if opts.maskEmail:
			# Note: This pattern masks more than emails.
			vPattern = re.compile(r"\b[^\s@]+@[\w\d].*?([\s,]|\.\s)", re.IGNORECASE)
			vInput, vNum1 = vPattern.subn(r" -EML-\1", vInput)
			vTotalEmails += vNum1
			if vNum1 > 0:
				vEmailLines += 1

		# masking URL
		if opts.maskURL:
			vPattern = re.compile(r"(t?ftp:|https?:).*?([\s#)\"',\]\}]|\.[\s])", re.IGNORECASE)
			vInput, vNum1 = vPattern.subn(r" -URL- \2", vInput)
			vTotalURLs += vNum1
			vPattern = re.compile(r"[w]{2,}\..*?\..*?([\s#)\"',\]\}]|\.[\s])", re.IGNORECASE)
			vInput, vNum2 = vPattern.subn(r" -URL- \1", vInput)
			vTotalURLs += vNum2
			vPattern = re.compile(r"\b[\S]{2,}[.](?:com|net|org|co\.uk|ru|ch|de|fr)\b[\S]*?([\s#)\"',\]\}]|\.[\s])", re.IGNORECASE)
			vInput, vNum3 = vPattern.subn(r" -URL- \1", vInput)
			vTotalURLs += vNum3
			if vNum1 + vNum2 + vNum3 > 0:
				vURLLines += 1
				vNum1 = 0
				vNum2 = 0
				vNum3 = 0

		# masking windows file paths
		if opts.maskWPath:
			## This pattern treats paths containing whitespace. The results
			## will be consumed by the next patterns to fully mask the path.
			## Note that it's not included in counting.
			vPattern = re.compile(r"\\[A-Za-z0-9_-]+(?:[\s]+[A-Za-z0-9_-]+){1,3}(?=\\)")
			vInput, vNum1 = vPattern.subn(r"\\WFP", vInput)
			# mathcing some known whitespace issues not captured above
			vPattern = re.compile(r"\\enterprise vault", re.IGNORECASE)
			vInput, vNum2 = vPattern.subn(r"\\WFP", vInput)
			# this pattern utilizes "" to handle paths containing whitespace
			#vPattern = re.compile(r"\s\"[A-Za-z]:[^\s]*?\\+.*?\"")
			vPattern = re.compile(r"[\"'][^\s]*?\\+.*?[\"']")
			vInput, vNum3 = vPattern.subn(r" -WFP- ", vInput)
			vTotalWPaths += vNum3
			vPattern = re.compile(r"(?:\b|\\)[\\]*(?:[^\s/]+?\\+)+[^\s/]*\b")
			vInput, vNum4 = vPattern.subn(r" -WFP- ", vInput)
			vTotalWPaths += vNum4
			## This pattern is meant to match file names. However, it may 
			## also match domain names with are missed above in URL masking
			## as well as some fused period. Also, it does not cover any 
			## possible file name.
			vPattern = re.compile(r"(['\"\(\s])[\w\d-]+\.[\w\d-]+(['\"\)\s])")
			vInput, vNum5 = vPattern.subn(r"\1-WFP-\2", vInput)
			vTotalWPaths += vNum5
			if vNum3 + vNum4 + vNum5 > 0:
				vWPathLines += 1
				vNum1 = 0
				vNum2 = 0
				vNum3 = 0
				vNum4 = 0
				vNum5 = 0

		# masking Unix-like file paths
		if opts.maskUPath:
			## This pattern treats paths containing whitespace. The results
			## will be consumed by the next patterns to fully mask the path.
			## Note that it's not included in counting.
			vPattern = re.compile(r"/[A-Za-z0-9_-]+(?:[\s]+[A-Za-z0-9_-]+){1,3}(?=/)")
			vInput, vNum1 = vPattern.subn(r"/UFP", vInput)
			# this pattern utilizes "" to handle paths containing whitespace
			#vPattern = re.compile(r"\s\"[A-Za-z]:[^\s]*?\\+.*?\"")
			vPattern = re.compile(r"[\"'][\S]*?/+.*?[\"']")
			vInput, vNum2 = vPattern.subn(r" -UFP- ", vInput)
			vTotalUPaths += vNum2
			vPattern = re.compile(r"(?:\b|/)[/]*(?:[^\s/]+?/+)+[^\s/]*\b")
			vInput, vNum3 = vPattern.subn(r" -UFP- ", vInput)
			vTotalUPaths += vNum3
			if vNum2 + vNum3 > 0:
				vUPathLines += 1
				vNum1 = 0
				vNum2 = 0
				vNum3 = 0

		# masking IP addresses and version number
		## Note: Since it is not easy to distinguish between IP and version,
		## they are masked together.
		if opts.maskIPV:
			vPattern = re.compile(r"\b[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}(?::[\d]{1,5})?(?:/[\d]{1,2})?(?=[\D])")
			vInput, vNum1 = vPattern.subn(r"-IPV-", vInput)
			vTotalIPVs += vNum1
			if vNum1 > 0:
				vIPVLines += 1
				vNum1 = 0

		# masking date/times
		## Note: Date and time are masked separately. They'll be merged 
		## later, when all neighbouring similar masked are merged in a 
		## post-processing run.
		if opts.maskDateTime:
			# times
			vTimeZones = "GMT|UTC|BST|CET|CEST|CST|EDT|ET|EST|EET|PDT|PST|UK"
			vTimePeriod = "AM|A\.M\.|A\.M|PM|P\.M\.|P\.M"
			#vPattern = re.compile(r"(?:(?<=[^\d:])|^)(?:(?:" + vTimeZones + r")[\+\-]?)?[\d]{1,2}:[\d]{2}(?::[\d]{2})?(?:\.[\d]+)?\s*(?:" + vTimePeriod + r")?\s*(?:(?:[\(](?:" + vTimeZones + r")[\)])|(?:" + vTimeZones + r"))?(?:\(mm:ss\))?(?=[^\d])", re.IGNORECASE)
			vPattern = re.compile(r"(?:(?<=[^\d:])|^)(?:(?:" + vTimeZones + r")[\+\-]?)?[\d]{1,2}[.:][\d]{2}(?:h(rs)?)?(?::[\d]{2})?(?:\.[\d]+)?\s*(?:" + vTimePeriod + r")?\s*(?:(?:[\(](?:" + vTimeZones + r")[\)])|(?:" + vTimeZones + r"))?(?:\(mm:ss\))?(?=[^\d])", re.IGNORECASE)
			vInput, vNum1 = vPattern.subn(r"-DTM- ", vInput)
			vTotalDTs += vNum1
			vPattern = re.compile(r"(?:(?<=[^\d:])|^)(?:(?:" + vTimeZones + r")[\+\-]?)?[\d]{1,2}(?:[:.][\d]{1,2})?\s*(?:" + vTimePeriod + r")\s*(?:(?:[\(](?:" + vTimeZones + r")[\)])|(?:" + vTimeZones + r"))?(?=[^\d\w])", re.IGNORECASE)
			vInput, vNum2 = vPattern.subn(r"-DTM- ", vInput)
			vTotalDTs += vNum2
			# dates
			vMonths = "January|Jan|February|Feb|March|Mar|April|Apr|May|June|Jun|July|Jul|August|Aug|September|Sep|October|Oct|November|Nov|December|Dec"
			vDays = "Monday|Mon|Tuesday|Tues|Tue|Wednesday|Wed|Thursday|Thurs|Thur|Thu|Friday|Fri|Saturday|Sat|Sunday|Sun"
			# e.g. 2/1/98, 02-1-1987
			vPattern = re.compile(r"\b\d{1,2}[/-]\d{1,2}[/-](?:\d{4}|\d{2})", re.IGNORECASE)
			vInput, vNum3 = vPattern.subn(r"-DTM- ", vInput)
			vTotalDTs += vNum3
			# e.g. 2 Jan 98, 02 December 2005
			vPattern = re.compile(r"\b\d{1,2}(?:th|st|rd|nd)?\s(?:" + vMonths + r")\s*[,]?\s*(?:\d{4}|\d{2})", re.IGNORECASE)
			vInput, vNum4= vPattern.subn(r" -DTM- ", vInput)
			vTotalDTs += vNum4
			# e.g. 2000-01-1, 98/11/10
			vPattern = re.compile(r"\b(?:\d{4}|\d{2})[/-]\d{1,2}[/-]\d{1,2}")
			vInput, vNum5 = vPattern.subn(r" -DTM- ", vInput)
			vTotalDTs += vNum5
			# e.g. December 31th, 2008 - Aug 18 
			vPattern = re.compile(r"(?:" + vMonths + r")[,]?\s+\d{1,2}(?:th|st|rd|nd)?[ \t]*[,]?[ \t]*(?:\d{4}|\d{2})?", re.IGNORECASE)
			vInput, vNum6 = vPattern.subn(r" -DTM- ", vInput)
			vTotalDTs += vNum6
			# e.g. Thursday 30th September 
			vPattern = re.compile(r"(?:" + vDays + r")[,]?\s+\d{1,2}(?:th|st|rd|nd)?[ \t]*[,]?[ \t]*(?:" + vMonths + r")?", re.IGNORECASE)
			vInput, vNum7 = vPattern.subn(r" -DTM- ", vInput)
			vTotalDTs += vNum7
			if vNum1 + vNum2 + vNum3 + vNum4 + vNum5 + vNum6 + vNum7 > 0:
				vDTLines += 1
				vNum1 = 0
				vNum2 = 0
				vNum3 = 0
				vNum4 = 0
				vNum5 = 0
				vNum6 = 0
				vNum7 = 0
				
			# merging neighbouring DTMs
			# e.g. -DTM- 2008
			vPattern = re.compile(r"(?:-DTM-\s*)+\s*\d{4}", re.IGNORECASE)
			vInput = vPattern.sub(r"-DTM- ", vInput)
			# e.g. Sunday, -DTM-
			vPattern = re.compile(r"(?:" + vDays + "),?\s*-DTM-", re.IGNORECASE)
			vInput = vPattern.sub(r" -DTM-", vInput)
			# e.g. 2008--DTM- T-DTM- Z
			vPattern = re.compile(r"(?:\b\d{4}-)?-DTM-\s*T\s*-DTM-[ \t]*(?:Z|\+-DTM-|--DTM-)?", re.IGNORECASE)
			vInput = vPattern.sub(r" -DTM- ", vInput)
			# e.g. 27- -DTM-
			vPattern = re.compile(r"\b\d{1,4}-[ \t]*-DTM-", re.IGNORECASE)
			vInput = vPattern.sub(r" -DTM-", vInput)
			# e.g. -DTM- -28, 2005
			vPattern = re.compile(r"-DTM-[ \t]*-\d{1,4},?[ \t]*(?:\d{1,4})?", re.IGNORECASE)
			vInput = vPattern.sub(r"-DTM- ", vInput)


		# writing final output
		# writing final output
		print vInput,
		vLineCntr += 1

	sys.stderr.write(str(vLineCntr) + " lines processed\n")			
	sys.stderr.write(str(vTotalHSpecials) + " HTML special characters in " + str(vHSpecialLines) + " lines masked\n")
	sys.stderr.write(str(vTotalSStrs) + " special strings in " + str(vSStrLines) + " lines masked\n")
	sys.stderr.write(str(vTotalRegKeys) + " windows registry keys in " + str(vRegKeyLines) + " lines masked\n")
	sys.stderr.write(str(vTotalPHolders) + " place-holders in " + str(vPHolderLines) + " lines masked\n")
	sys.stderr.write(str(vTotalLNums) + " list numbers in " + str(vLNumLines) + " lines masked\n")
	sys.stderr.write(str(vTotalMItems) + " menu paths in " + str(vMItemLines) + " lines masked\n")
	sys.stderr.write(str(vTotalTags) + " HTML/XML tags in " + str(vTagLines) + " lines masked\n")
	sys.stderr.write(str(vTotalVars) + " variables in " + str(vVarLines) + " lines masked\n")
	sys.stderr.write(str(vTotalBSlashs) + " newline/tab/.. in " + str(vBSlashLines) + " lines masked\n")
	sys.stderr.write(str(vTotalEmails) + " Emails in " + str(vEmailLines) + " lines masked\n")
	sys.stderr.write(str(vTotalURLs) + " URLs in " + str(vURLLines) + " lines masked\n")
	sys.stderr.write(str(vTotalWPaths) + " Windows file pathes in " + str(vWPathLines) + " lines masked\n")
	sys.stderr.write(str(vTotalUPaths) + " Unix file pathes in " + str(vUPathLines) + " lines masked\n")
	sys.stderr.write(str(vTotalIPVs) + " IP addresses and version numbers in " + str(vIPVLines) + " lines masked\n")
	sys.stderr.write(str(vTotalDTs) + " date/times in " + str(vDTLines) + " lines masked\n")

##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())

