#! /usr/bin/python

## This script replaces all tokens longer than N characters in input file
## with a mask string 
##
## Current version: 1.0
##


import sys, optparse


##----------------------------------------------------------------------
## Mask the pString by pMaskStr and considers the options for bracketing. 
## If bracketing options are false, the result is simply pMaskStr.
## Bracketing options are usefull when dealing with parse trees.
def maskString(pString, pMaskStr, pKeepBeginBrackets, pKeepEndBrackets):
 vBeginBracketStr = ""
 vEndBracketStr = ""

 if pKeepBeginBrackets:
  for vChar in pString:
   if vChar in ['(', '{', '[']:
    vBeginBracketStr += vChar
   else:
    # stop as soon as a non-bracket character was observed
    break
   
 if pKeepEndBrackets:
  for vChar in reversed(pString):
   if vChar in [')', '}', ']']:
    vEndBracketStr = vChar + vEndBracketStr
   else:
    # stop as soon as a non-bracket character was observed
    break
   
 return vBeginBracketStr + pMaskStr + vEndBracketStr


##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv
 
 parser = optparse.OptionParser(usage="%prog <INPUT FILE> <WORD LENGTH LIMIT> <MASK STRING> [OPTIONS]" + 
                                      "\nReplaces all string longer than N words in input file with a mask string.", version="%prog 1.0")

 parser.add_option("-b", help="keep consequtive bracketing at the beginning of long tokens", dest="keepBeginBracket", action="store_true")
 parser.add_option("-e", help="keep consequtive bracketing at the end of long tokens", dest="keepEndBracket", action="store_true")


 # processing input arguments

 (opts, posArgs) = parser.parse_args()
 
 if len(posArgs) < 2:
  parser.error("At least 2 arguments are required")
  
 
 try:
  vfInput  = open(posArgs[0], 'r')
 except IOError:
  sys.exit('Can\'t open input file: ' + posArgs[0])

 vLengthLimit = int(posArgs[1])

 if len(posArgs) > 2:
  cMaskStr = posArgs[2]
 else:
  cMaskStr = "LongString" 


 # processing content
  
 for vLine in vfInput:
  vlTokens = vLine.strip().split()
  vNewLine = ""
  for vToken in vlTokens:
   if len(vToken) > vLengthLimit:
    vNewLine += maskString(vToken, cMaskStr, opts.keepBeginBracket, opts.keepEndBracket) + ' '
   else:
	vNewLine += vToken + ' '
	
  print vNewLine[:-1]
   
 vfInput.close();
 
 

##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())
