#! /usr/bin/python

## This script gets a language model, a test set, and computes the perplexity of contiguous blocks of text in test set with respect to that langauge model.
## Current version: 1.0
##
## The IRSTLM compile-lm program should be provided as an argument to this script.
## The output is the perplexities of each line of the test set each in a new lines. 
## A block is a fixed number of sentences. The number is provided as an option to this script. The test set is divided into blocks based
## on this number. 

import sys, optparse, os
from subprocess import Popen, PIPE

##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv
 
 parser = optparse.OptionParser(usage="%prog [-t TEST DATA] [-l LANGUAGE MODEL] [-c IRSTLM COMPILE-LM] [-n BLOCK SIZE]" + 
                                      "\nGets a language model, a test set, and computes the perplexity of contiguous blocks of " + 
                                      "text in test set with respect to that langauge model.", version="%prog 1.0")

 parser.add_option("-t", "--test", help="test data set", metavar="TEST DATA", dest="testFileName", action="store")
 parser.add_option("-l", "--lm", help="language model", metavar="LANGUAGE MODEL", dest="lmFileName", action="store")
 parser.add_option("-c", "--compilelm", help="IRSTLM compile-lm", metavar="IRSTLM COMPILE-LM", dest="compileLM", action="store")
 parser.add_option("-n", "--blocksize", help="Number of sentences in each block", metavar="BLOCK SIZE", dest="blockSize", action="store", type="int", default="1")

 (opts, posArgs) = parser.parse_args()
 
 # opening test file
 if opts.testFileName == None:
  print 'Test file is not provided!'
  sys.exit(2)
 else:
  try:
   testFile  = open(opts.testFileName, 'r')
  except IOError:
   print 'Can\'t open test: ' + opts.testFileName
   sys.exit(2)

 vEOF = False
 while not vEOF:
  # counter for controling the block size
  vCntr = 0 

  # Compile-lm accept the test input as a file. Sentence are written to a temporary file. The file is overwriten per sentence.
  try:
   tmpInput = open("tmpinput", 'w')
  except IOError:
   print 'Can\'t create temporary input file in the current path'
   sys.exit(2)
  
  vBlock = ""
  
  # write end tag at the beginning of the block (IRSTLM convention)
  #tmpInput.write("</s>\n")

  vLine = testFile.readline()
  while (vLine):
   # append sentence to block
   vBlock += "<s> " + vLine.splitlines()[0].strip() + " </s>\n"
   #tmpInput.write("<s> " + vLine.splitlines()[0].strip() + " </s>\n")
   
   vCntr += 1
   if vCntr == opts.blockSize:
    break
   else:
    vLine = testFile.readline()

  ## If line is empty, eof has reached. So, set the eof flag for the main 
  ## loop uses and if there is already something
  ## in block, use it. Otherwise, finish. 
  if (not vLine):
   vEOF = True
   if (not vBlock):
    break

  # write end/start tag at the beginning/end of the block (IRSTLM convention)
  vBlock = "</s>\n" + vBlock + "<s>\n"

  ## writing the block into caculator's input file#
  tmpInput.write(vBlock)
  tmpInput.close()
  
  # calling perplexity calculator
  p1 = Popen([opts.compileLM, "--eval=./tmpinput", opts.lmFileName], stdout=PIPE)
  p2 = Popen(["grep", "%%"], stdin=p1.stdout, stdout = PIPE)
  p1.stdout.close()
  vPPLine = p2.communicate()[0]
  p1.wait()
  
  # processing the output to fetch perplexity value
  vPPPos = vPPLine.find(" PP=")
  if vPPPos == -1:
   print "Perplexity score wasnot found in the output of calculator"
   sys.exit(2)
  else:
   vPPLStr = vPPLine[vPPPos + 4:].split()[0]
   try:
    vPPL = float(vPPLStr)
   except ValueError, TypeError:
	print ("Invalid float value")
   else:
    print vPPLStr
 
 
 testFile.close()
 os.remove(tmpInput.name) 


 
##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())


