#! /usr/bin/python

## This script extracts parse trees from Berkeley output which are arranged 
## as split packages with fixed sizes (except the last one).
##
## Current version: 1.1
##
## Changes:
## - Adding TOP node is made optional to handle differnet Berkeley output 
##   resulted from different grammar (e.g. German grammar outputs TOP by
##   default)

from operator import itemgetter
import sys, optparse, berkeleyparses


##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv
 
 parser = optparse.OptionParser(usage="%prog <PACKAGES FILES NAME PATTERN>" + 
                                      "\nExtracts parse trees from Berkeley output arranged as fixed-size split packages.", version="%prog 1.0")
 
 parser.add_option("-d", "--dummy", help="type of dummy parse for unparsable sentences (x: X-labeled; u: UNPARSED)", metavar="DUMMAY PARSE TYPE", dest="dummyParseType", default='u', action="store")
 parser.add_option("-r", "--rawinput", help="raw input file to be used with -d x to extract raw sentences for generating dummy parse", metavar="RAW INPUT FILE", dest="rawInputFileName", action="store")
 parser.add_option("-e", "--encoding", help="encoding type of input and output", metavar="ENCODING", dest="encodingType", action="store")
 parser.add_option("-t", help="add TOP node", metavar="TOP NODE", dest="addTOP", action="store_true")

 # processing input arguments

 (opts, posArgs) = parser.parse_args()
 
 if len(posArgs) < 1:
  parser.error("At least 1 arguments are required")
 
 # sorting packages by name
 vlPackages = posArgs
 vlPackages.sort()
 
 
 # extracting parses from all packages.
 
 for vPackage in vlPackages:
  sys.stderr.write(vPackage + '\n')
  
  if opts.dummyParseType == 'x':
   sys.exit("X-labeled dummy parse generation has not been implemented yet. Please chose u.")
   ### to be implemented
   #if opts.rawInputFileName == None:
   # vfOutput.close()
   # sys.exit("Raw input file is not specified to generate X-tagged dummy parses!")
   #else:
   # vlCurrPkgParse = berkeleyparses.extractParses(vPackage, True, True, opts.rawInputFileName)
  elif opts.dummyParseType == 'u':
   berkeleyparses.extractParses(vPackage, opts.encodingType, opts.addTOP, True, True, False, True)


##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())

