#! /usr/bin/python

## This script identifies N best and M worst parses from an input pasre
## tree file based on provided evalb score file. Any of recall, precesion
## and F-score can be chosen as comparision measure.
## It also gets the corresponding tokenized raw text. The outputs are four 
## files, two containing best and worst parses and the others best + worst 
## raw sentences. In addition, prints the number of the extracted sentences 
## according to their original order.
##
## Current version: 1.1
##
## Added:
## - Outputs selected sentences numbers (their order in the input) into 
##   stdout
## - Output files are optionally named with provided prefix.
##

from operator import itemgetter
import sys, optparse, evalbproc

##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv
 
 parser = optparse.OptionParser(usage="%prog <PARSE TREE FILE> <TOKENIZED RAW TEXT FILE> <EVALB SCORE FILE> <OUTPUT DIR> [options]" + 
                                      "\nExtracts N best and M worst parses from an input pasre tree file based on provided scores.", version="%prog 1.1")

 parser.add_option("-b", "--bestnum", help="number of best parses to be extracted", dest="bestNum", type='int', default=0, action="store")
 parser.add_option("-w", "--worstnum", help="number of worst parses to be extracted", dest="worstNum", type='int', default=0, action="store")
 parser.add_option("-s", "--sortscore", help="score to be compared: (r)ecall, (p)recision or (f)-score)", dest="sortScore", type='str', default='f', action="store")
 parser.add_option("--bestprefix", help="prefix for best output files", dest="bestPrefix", type='str', default='best-parse', action="store")
 parser.add_option("--worstprefix", help="prefix for worst output files", dest="worstPrefix", type='str', default='worst-parse', action="store")

 (opts, posArgs) = parser.parse_args()
 
 if len(posArgs) < 4:
  parser.error("At least 4 argument is required")
 
 # processing options
 
 if opts.bestNum == 0 and opts.worstNum == 0:
  print "No best and worst parse number is specified. The program exits peacefully!"
  sys.exit(0)
  
 ## assuming that the recall, precision and f-score have these orders 
 ## in the Scores namedtuple in evalbproc.py
 if opts.sortScore == 'r':
  vSortScore = 1
 elif opts.sortScore == 'p':
  vSortScore = 2
 else:
  vSortScore = 3
  
  
 # loading scores from file
   
 vEvalbFileName = posArgs[2]
 vScores = evalbproc.extractSentenceScores(vEvalbFileName, False, False, False, False)
 
 if opts.bestNum + opts.worstNum > len(vScores):
  sys.exit('Number of best+worst is greater than the number of sentences!')
 
 # sorting the scores
 vScores.sort(key=itemgetter(vSortScore), reverse=True)
 
 # sub-selecting the scores for the corresponding N best and M worst parses
 vBestScores = vScores[:opts.bestNum]
 if opts.worstNum > 0:
  vWorstScores = vScores[-opts.worstNum:]
 else:
  vWorstScores = []

 # loading parses

 vParseFileName = posArgs[0]
 try:
  vfParse  = open(vParseFileName, 'r')
 except IOError:
  sys.exit('Can\'t open parse file: ' + vParseFileName)

 vlParses = []

 for line in vfParse:
  vlParses.append(line)

 vfParse.close()


 # loading tokenized sentences

 vTextFileName = posArgs[1]
 try:
  vfText  = open(vTextFileName, 'r')
 except IOError:
  sys.exit('Can\'t open evalb file: ' + vTextFileName)

 vlSentences = []

 for line in vfText:
  vlSentences.append(line)

 vfText.close()
 
 
 # creating output files
 
 if opts.bestNum != 0:
  try:
   vfBestParses  = open(posArgs[3] + '/' + opts.bestPrefix + '.' + str(opts.bestNum), 'w')
  except IOError:
   print 'Can\'t create best parse output file in ' + posArgs[3]
   sys.exit(2)

  try:
   vfBestSentences  = open(posArgs[3] + '/' + opts.bestPrefix + '.sntns.' + str(opts.bestNum), 'w')
  except IOError:
   print 'Can\'t create best parse output file in ' + posArgs[3]
   sys.exit(2)

 if opts.worstNum != 0:
  try:
   vfWorstParses  = open(posArgs[3] + '/' + opts.worstPrefix + '.' + str(opts.worstNum), 'w')
  except IOError:
   print 'Can\'t create worst parse output file in ' + posArgs[3]
   sys.exit(2)

  try:
   vfWorstSentences  = open(posArgs[3] + '/' + opts.worstPrefix + '.sntns.' + str(opts.worstNum), 'w')
  except IOError:
   print 'Can\'t create worst parse output file in ' + posArgs[3]
   sys.exit(2)


 # extracting best/worst parses according to the sorted scores list
 
 print "Selected best parses:"
 for scores in vBestScores:
  print scores.no
  try:
   vfBestParses.write(vlParses[scores.no - 1])
  except NameError:
   pass
  try:
   vfBestSentences.write(vlSentences[scores.no - 1])
  except NameError:
   pass

 print "Selected worst parses:"
 for scores in vWorstScores:
  print scores.no
  try:
   vfWorstParses.write(vlParses[scores.no - 1])
  except NameError:
   pass
  try:
   vfWorstSentences.write(vlSentences[scores.no - 1])
  except NameError:
   pass
  
 
 try:
  vfBestParses.close()
 except NameError:
  pass

 try:
  vfBestSentences.close()
 except NameError:
  pass

 try:
  vfWorstParses.close()
 except NameError:
  pass

 try:
  vfWorstSentences.close()
 except NameError:
  pass


##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())

