#! /usr/bin/python

## This script extracts parse trees from LORG output which are arranged 
## as split packages with fixed sizes (unless the last one).
##
## Current version: 1.0
##

from operator import itemgetter
import sys, optparse, lorgparses, codecs


##======================================================================
## main
def main(argv=None):
 if argv is None:
  argv = sys.argv
 
 parser = optparse.OptionParser(usage="%prog <PACKAGE SIZE> <CORPUS SIZE> <OUTPUT FILE NAME> <PACKAGES FILES NAME PATTERN>" + 
                                      "\nExtracts parse trees from LORG output arranged as fixed-size split packages.", version="%prog 1.0")

 # processing input arguments

 (opts, posArgs) = parser.parse_args()
 
 if len(posArgs) < 4:
  parser.error("At least 4 arguments are required")
 
 vPackageSize = int(posArgs[0])
 vCorpusSize = int(posArgs[1])
 
 # creating output file
 try:
  vfOutput  = codecs.open(posArgs[2], mode='w', encoding='utf-8')
 except IOError:
  sys.exit('Can\'t create output file: ' + posArgs[2])

 
 # sorting packages by name
 vlPackages = posArgs[3:]
 vlPackages.sort()
 
 # extracting parses from all packages.
 
 vlParses = []
 vPackCntr = 0
 for vPackage in vlPackages:
  print vPackage
  vPackCntr += 1
  vlPackageParses = []
  
  vlPackageParses += lorgparses.extractParses(vPackage, True, False)
  
  ## adjusting sentence numbers based on package number (packages are sorted
  ## above)
  for vParseTree in vlPackageParses:
   if vParseTree.sentenceNo > vPackageSize:
    sys.exit("Sentence number exceeds the package size! package: " + vPackage)
   else:
    vlParses.append(lorgparses.Parse(vPackageSize * (vPackCntr - 1) + vParseTree.sentenceNo, vParseTree.tree))

  vlPackageParses = []
  
    
 # inserting dummy parse trees for missing sentences
 vDummyParse = '(TOP (UNPARSED UNPARSED))'
 vTreeCntr = 1

 # this first loop treats missing trees from the middle
 while (vTreeCntr <= vlParses[-1].sentenceNo):
  if vlParses[vTreeCntr - 1].sentenceNo > vTreeCntr:
   vlParses.insert(vTreeCntr - 1, lorgparses.Parse(vTreeCntr, vDummyParse))
  elif vlParses[vTreeCntr - 1].sentenceNo < vTreeCntr:
   sys.exit("Unknown error: sentence match problem")

  vfOutput.write(vlParses[vTreeCntr - 1].tree.decode('UTF-8') + '\n')
  vTreeCntr += 1

 # this last loop treats missing trees from the end
 while (vTreeCntr <= vCorpusSize):
  vlParses.append(lorgparses.Parse(vTreeCntr, vDummyParse))

  vfOutput.write(vlParses[vTreeCntr - 1].tree.decode('UTF-8') + '\n')
  vTreeCntr += 1

 vfOutput.close()   
 

##======================================================================
## calling main
if __name__ == "__main__":
 sys.exit(main())
