#! /usr/bin/python
# -*- coding: utf-8 -*-

"""	
	This module defines functions and data structures related to parsing.
	
	Version 5.3												(25-May-2015)
	- ConstParses, ConstParse, ConstTree, ConstNode, BConstParser are moved
	  to constparse.py.
	- DepTree, DepNode, DepEval are moved to depparse.py.
	
	Version 5.2												(05-May-2015)
	- POSTagging and POSTaggedToken are moved to pos.py.
	
	Version 5.1												(30-Apr-2015)
	- toLorgInput(), toStanfTSVInput(), toSVMToolInput() are added to 
	  POSTagging and POSTaggedToken.
	- replaceChild(), insertParent() are added to ConstNode.
	- insertChild() is split into two methods: itself and insertIntermChild()
	  Both now return the inserted child node.
	
	Version 5.0												(18-Feb-2015)
	- prepForParsing() is improved to mask characters at the beginning of
	  line stoping Lorg.
	- ConstTree.loadPTBTree() and ConstNode._creadtPTBNode() are edited 
	  and ConstTree._createRoot() and ConstNode._getNewNode() are edited
	  to abstract away creating a node.
	- insertChild() is added to ConstNode. Compared to the existing addChild() 
	  which simply add a child under the node, it can manipulate other 
	  children.
	- setLabel() and getLabel() are added to ConstNode. Label is equivalent
	  to syntactic tag (synTag) which is introduced as a naming change.
	  getLabel() is thus doing the same as getSynTag(). getSynTag() is now
	  calling getLabel() to get the value.  
	- getChildrenAt(), delChildrenAt(), setParent(), setExpandedTerminal()
	  and getExpandedTerminal() are added to ConstNode.
	
	Version 4.9												(08-Dec-2014)
	- POSTagging.loadPTBPOSTagging() is tested and fixed and is ready to
	  be used.
	
	Version 4.8												(07-Oct-2014)
	- Dependency relation case is added to annotation options in 
	  _loadCoNLL200xAnnotation() of DepNode.
	
	Version 4.7												(19-Sep-2014)
	- hasSuffixedLabel(), delPRO(), getPreTerminalNodes() are added to 
	  ConstNode and ConstTree.
	- ConstNode.delChild() is edited to delete nodes which are left without 
	  a child after child deletion.
	- IMPORTANT: ConstNode.getChildren() is edited to return a copy of list 
	  of children instead of the list itself. THE EFFECT ON THE PREVIOUSLY-
	  WRITTEN METHODS USING IT IS NOT TESTED.
	
	Version 4.6												(15-Aug-2014)
	- removeSuffix() and delNodes() are added to ConstNode and ConstTree.
	- Bug in ConstTree.surface() is fixed.
	
	Version 4.5												(14-May-2014)
	- DepTree.loadCoNLLDependency() is updated to use getSRL() method of
	  CoNLL sentence instead of its srl property.
	
	Version 4.4												(24-Apr-2014)
	- POSTagToken is renamed to POSTaggedToken and updated to load from 
	  SlashTag format.
	- toCoNLLParserInput() is added to POSTaggedToken and POSTagging.
	- mapTag() is added to POSTaggedToken.
	- mapTagging() is added to POSTagging.
	
	Version 4.3												(04-Apr-2014)
	- Language is added to DepTree attributes.
	- toCoNLL2009Sentence(), getNodes(), getSRLPropCount(), getPredOrdinal()
	  are added to DepTree.
	- getHeadDeps(), getPredArgRoles() are added to DepNode.
	- Bug is fixed in DepTree.loadCoNLLDependency() which was loading SRL
	  only if onSRLConstraintViolation option was set.
	- Bug in DepTree.loadSRL() is fixed to remove the current SRL annotation
	  from nodes before loading a new one.
	
	Version 4.2												(11-Mar-2014)
	- Bug in DepEval._loadTrees() is fixed to handle PARC700 trees correctly.
	- Bug in DepTree.compareWith() is fixed to ignore MISSINGHEAD of Const2Dep
	  output for French trees.
	- pflgCheckWordMismatch and pflgCaseSensitive are added to DepTree.compareWith()
	  and DepEval.score() arguments.
	- isMISSINGHEAD() is added to DepNode.
	
	Version 4.1												(11-Mar-2014)
	- Language is added to DepEval.
	- Annotation option is added to parse loading methods of DepEval.
	
	Version 4.0												(26-Feb-2014)
	- getSibling() is added to ConstNode.
	- ConstTree.loadSRL() is edited to adapt to changes in SRL.convertToConst()
	  method. Now, instead of loading the SRL into tree after conversion,
	  the loading takes place during conversion.
	- hasPredRole(), addPredRole(), removePredRole() are added to ConstNode.
	- isVerb() and getDepPositions() are added to DepNode.
	
	Version 3.9												(11-Feb-2014)
	- surface() is added to ConstTree and ConstNode.
	- DepParseLoader is moved to dloader.
	- DepTree.loadCoNLLDependency() is updated to consider onSRLConstraintViolation
	  option when loading SRL.
	- getSentLength() and surface() are added to DepTree.
	
	Version 3.8												(30-Jan-2014)
	- getPST() is added to ConstTree
	- getRawPST() is added to ConstNode.
	- ConstNode.deepCopy() is edited to include copying the attributes 
	  added since.
	- shallowCopy(), addChild(), isArgumentOf(), getArgRoleOf(), ifDominates(),
	  setPredRoles(), removePredRoles() are added to ConstNode.
	- ConstTree.loadSRL() is edited to take care of duplicate argument roles
	  for a same predicate.
	- depCoNLL2PTBTree() is update with new CoNLLDataLoader format.
	
	Version 3.7												(24-Jan-2014)
	- A bug in ConstNode.getTokenSpan() is fixed. Instead of computing
	  the span in the sentence, it wrongly computed it in the span of
	  parent node.
	- isRoot(), getRoot(), getGrandParentUnderRoot(), getSynTag(), getChildren(),
	  getTerminal() are added to ConstNode.
	- getPreTerminalNodeAt() is added to ConstTree.
	- A bug in ConstNode.getPreTerminalNodeAt() is fixed. It no more returns
	  a terminal as pre-terminal for collapsed terminal.
	
	Version 3.6												(21-Jan-2014)
	- ConstNode.predRoles is added.
	- getTokenSpanRelation(), getLowestSpanNode(), getParent(), 
	  getPreTerminalNodeAt() are added to ConstNode.
	- getLowestSpanNode(), getPreTerminalNodeAt(), getArgRoles(), 
	  getPredicateFrameset(), isPredicate(), isArgument() are added to 
	  ConstTree.
	- ConstParseLoader is moved to dloader.py.
	
	Version 3.5												(17-Jan-2014)
	- DepNode.getDepRel() is upgraded to optionally return the dependency
	  relation to a given head for multihead nodes instead of the first 
	  one as default.
	- getArgRoles(), getPredicateFrameset(), isPredicate(), isArgument() 
	  are added to DepNode.
	
	Version 3.4												(14-Jan-2014)
	- getSpanSurface() and getNode() are added to DepTree.
	- getForm(), getDepRel(), getPOSTag(), getHeadNode(), isRoot() added 
	  to DepNode.
	- DepTree.__str__() is changed from PTB bracketing representation to
	  node-by-node representation.
	- depTree is added as attribute to DepNode. getDependentNodes(), 
	  getDepDepRels(), getHeight() are accordingly adjusted by removing 
	  pDepTree parameter previously passed into them.
	
	Version 3.3												(30-Dec-2013)
	- Predicate argument structure is added to dependency tree.
	- DepTree.loadCoNLLDependency() is upgraded to account for various
	  annotation based on CoNLL format version. Accordingly, internal 
	  methods are added to DepNode to load the format-specific annotations.
	
	Version 3.2												(21-Nov-2013)
	- CoNLL loader and PARC700 loader are moved to dloader.py.
	
	Version 3.1												(31-Oct-2013)
	- Bugs fixed in PARC700Token.__init__() and PARC700Sentence.__init__().
	
	Version 3.0												(02-Aug-2013)
	- ConstParseLoader.loadParses() is added.
	
	Version 2.9												(25-Jul-2013)
	- getFormRelPairs() is added to DepTree and DepNode.
	- prob() in ConstParses is bug fixed.
	
	Version 2.8												(08-Jul-2013)
	- ConstNode.isExpandedTerminal() is bug-fixed.
	- extractLexRules() is added to ConstTree and ConstNode.
	
	Version 2.7												(05-Jul-2013)
	- extractCFGRules() of ConstTree and ConstNode and ConstNode.getCFGRule()
	  optionally extract/ignore lexical rules.
	- addVPToFTBTree() is added to ConstNode and ConstTree. 
	- stratifyRules() is added to ConstNode and ConstTree and _stratifyPartialRule()
	  to ConstNode.
	- ConstNode.isCollapsedTerminal() is added.
	- ConstNode.fixFTBCoord() is added.
	
	Version 2.6												(01-Jul-2013)
	- expandedTerminal is added to ConstTree and ConstNode to support
	  pre-terminals by expanding terminal into pre-terminal and terminal
	  nodes. The involved methods in ConstParseLoader, ConstTree and ConstNode
	  are updated with a new parameter to opt for expanding the terminal
	  wile loading the parses, tree or constituent.
	- ConstNode.isPreTerminal() and  ConstNode.isExpandedTerminal() are 
	  added.
	- extractSubsetTrees() is added to ConstTree and ConstNode.
	- getCFGRules() is renamed to extractCFGRules().
	
	Version 2.5												(25-Jun-2013)
	- getDepRelTagset() of DepTree and DepParseLoader is improved to return
	  tag counts as well.
	- deepCopy() is added to ConstTree and ConstNode.
	 
	Version 2.4												(21-Jun-2013)
	- topNode(), getTopNodePOS(), getDepRelTagset(), height are added to
	  DepTree.
	- DepParseLoader is added.
	- getDepCount(), getDepDepRels(), getDependentNodes(), getHeight() 
	  are added to DepNode.
	
	Version 2.3												(10-Jun-2013)
	- PARC700Sentence's constructor is improved to handle double-MISSINGHEAD
	  situation in the output of const2dep.
	- getChildrenCount() is added to ConstNode.
	- getCFGRule() is added to ConstNode.
	- getCFGRules() is added to ConstNode  and ConstTree.
	
	Version 2.2												(07-Jun-2013)
	- ConstNode.mapTags() is improved to used catch-all tag.
	- getTagset() is added to ConstTree and ConstNode.
	
	Version 2.1												(03-Jun-2013)
	- ConstParseLoader and ConstParse are added.
	- extractLORGParses() is integrated into ConstParseLoader.
	- loadTagMap() is improved to optionally include or exclude self-maps
	  (e.g. NN to NN).
	- POSTagging() and POSTagToken() are added.
	- getUniversalPOSs() is added to ConstTree.
	
	
	Version 2.0												(15-May-2013)
	- extractLORGParses() is added.
	- BConstParser is added.
	- isTerminal() is added to ConstNode.
	
	Version 1.9												(08-May-2013)
	- getPOSs() and getPreTerminals() are added to ConstTree and ConstNode
	  respectively.
	- getCoarseChunks() is added to ConstTree.
	- ConstTree.mapTag() is added.
	- loadTagMap() is added.
	- ConstTree.getSentLength() is added.
	
	Version 1.8												(02-May-2013)
	- convertDQuoteWSJ() is moved from nlp.py.
	- convertSQuoteWSJ() is added.
	- prepForParsing() is improved to convert single quotes to WSJ.
	
	Version 1.7												(30-Apr-2013)
	- ConstTree and ConstNode are added.
	- DepEval is added.
	- compareWith() is added to DepTree.
	- selectSentences() is added to CoNLLLoader
	
	Version 1.6												(16-Apr-2013)
	- extractCTBTokens()
	
	Version 1.5												(04-Apr-2013)
	- CoNLLDataLoader is added.
	- CoNLLSentence is added.
	- DepTree and DepNode are added.
	- depCoNLL2PTBracketing() is added.
	- PARC700DataLoader is added.
	- PARC700Sentence is added.
	
	Version 1.4												(26-Mar-2013)
	- prepForParsing() and extractPOS() are moved from nlp.py
	- extractCTBTags() is added.
	- mapTags() is added.
	- assimilateTags() is added.
	- assimilateTerminals() is added.
	
	Version 1.3
	- extractTreeDepth()
	- extractConstituentCnt()
"""

import srl, nlp.nlp, dloader, constparse, depparse
import sys, random, re, math, copy, itertools

from utils import util


def extractTreeDepth(pTree): 
	'''
	Extracts the depth of parse tree
	
	It's the maximum number of consecutive opening bracket happening in 
	the tree (i.e. without any closing bracket in between).
	'''
	
	vTmpOpenCntr = 0
	vMaxOpen = 0
	for vChar in pTree:
		if vChar == '(':
			vTmpOpenCntr += 1
		elif vChar == ')':
			if vMaxOpen < vTmpOpenCntr:
				vMaxOpen = vTmpOpenCntr
		vTmpOpenCntr -= 1
	
	return vMaxOpen
	


def extractConstituentCnt(pTree): 
	'''
	Extracts number of constituents in parse tree
	
	It's simply the number of opening brackets in the tree.
	'''
	
	vOpenBCntr = 0
	for vChar in pTree:
		if vChar == '(':
			vOpenBCntr += 1
	
	return vOpenBCntr
	


def extractCTBTags(pTreebank, pflgWithFTags):
	'''
	Extracts tags including phrase categories and POS tags from a constituency
	Treebank in bracketed format and return a dictionary of tags and their
	frequency.
	
	If pflgWithFTags is set to true, the returnes tags will also include
	function tags (e.g. SQ-PRD).
	'''
	
	import re, util
	
	# extracting raw tags (including function tags and traces)
	vPattern = re.compile(r"\(([^() ]+)")
	vlRawTags = re.findall(vPattern, pTreebank)
	
	# processing raw tags to prune traces and function tags
	vlTags = []
	for vTag in vlRawTags:
		if pflgWithFTags:
			vPattern = re.compile(r"[-=]\d+")
			vlTags.append(vPattern.sub("", vTag))
		else:
			vPattern = re.compile(r"(?<!^)[-=]..*")
			vlTags.append(vPattern.sub("", vTag))
	
	return util.groupBy(vlTags)
	


def extractCTBTokens(pTreebank):
	'''
	Extracts tokens from a constituency Treebank in bracketed format.
	'''
	
	import re
	
	vlSents = []
	for vTree in pTreebank.strip().split('\n'):
		vlTokens = []
		vPattern = re.compile(r" ([^ )]+)\)")
		vlTokens = re.findall(vPattern, vTree)
		# pruning traces
		vlTokens = [i for i in vlTokens if i not in ["*T*", "*PRO*", "*EXP*"]]
		vlSents.append(' '.join(vlTokens))
	
	return '\n'.join(vlSents) + '\n'
	


def mapTags(pTrees, pMap, pDelim = '\t'):
	'''
	Replaces the tags in the input trees using the map given in pMap.
	
	pMap contains maps, one per line, each mapping a tag from source Treebank
	tag set to target Treebank tag set. Source trees should be in bracketed
	format.
	'''
	
	import re
	
	# loading map into a dictionary
	vdMap = loadTagMap(pMap, pDelim)
	
	# doing the replacement
	vTrees = pTrees
	vlCounts = []
	for vSrc, vTrg in vdMap.iteritems():
		# '('+vSrc+' ' Rsecures conflict between tags and possible similar words.
		vPattern = re.compile('\(' + vSrc + ' ', re.IGNORECASE)
		vTrees, vNum = vPattern.subn('(' + vTrg + ' ', vTrees)
 		vlCounts.append("%-16s\t%s" % (vSrc, vNum))
	
	return vTrees, '\n'.join(vlCounts)
	


def loadTagMap(pMap, pDelim = '\t', pflgSelfMapIn = True):
	'''
	Loads tag set map from pMap into a dictionary.
	
	It optionally includes or excludes self-maps (e.g. NN to NN).
	'''
	
	vdMap = {}
	for vMap in pMap.strip().split("\n"):
		if vMap.startswith("##"):
			continue
		# converting delimiters to space to handle trailing delimiters
		vMap.replace(pDelim, ' ')
		vlMapTags = vMap.split()
		if pflgSelfMapIn:
			vdMap[vlMapTags[0]] = vlMapTags[1]
		else:
			if vlMapTags[0] != vlMapTags[1]:
				vdMap[vlMapTags[0]] = vlMapTags[1]
	
	return vdMap
	
 
 
def assimilateTags(pTrees):
	'''
	Assimilates all the phrase labels nad POS tags in pTrees by replacing
	them with X
	'''
	
	import re
	
	vPattern = re.compile(r"\([^() ]+ ")
	return vPattern.subn("(X ", pTrees) 
	


def assimilateTerminals(pTrees):
	'''
	Assimilates all the tree terminals in pTrees by replacing them with w
	'''
	
	import re
	
	vPattern = re.compile(r" [^()]+\)")
	return vPattern.subn(" w)", pTrees) 
	


def prepForParsing(pCorpus, pStyle, pParser = "", pflgTokenize = False):
	'''
	Performs a set of preprocessing actions to prepare the corpus in pStyle
	style for parsing. The following pStyles are supported:
	
	-wsj: masking brackets, converting quotations, escaping slashes.
	'''
	
	## The following characters stop Lorg when located at the beginning of 
	## the line.
	cLorgBLMasks = {"!": "-XCL-",
	                "?": "-QSN-",
	                "#": "-HSH-"
					}
	
	vStyle = pStyle.lower()
	vParser = pParser.lower()
	
	vlOutput = []
	
	for vLine in pCorpus.strip().split('\n'):
		if vStyle in ["wsj", "ftb"]:
			# masking round brackets
			vLine, dummy = nlp.maskRBracket(vLine, pflgTokenize)
		
		if vParser in ["lorg"]:
			# masking square brackets for LORG (it omits them)
			vLine, dummy = nlp.maskSBracket(vLine, pflgTokenize)
			# masking special characters at the beginning of line which stop Lorg
			for vChar, vMask in cLorgBLMasks.iteritems():
				if vLine[0] == vChar:
					vLine = vMask + vLine[1:]
		
		if vStyle in ["wsj"]:
			# masking curly brackets
			vLine, dummy = nlp.maskCBracket(vLine, pflgTokenize)
		
		if vStyle in ["wsj"]:
			# escaping /
			vLine = vLine.replace('/', "\\/")
		
		if vStyle in ["wsj"]:
			# converting single quotations to ` '
			vLine = convertSQuoteWSJ(vLine)
			# converting double quotations to `` ''
			# It's important to do single quote first
			vLine = convertDQuoteWSJ(vLine)
		
		if vParser.startswith("berk"):
			# making @
			vLine, dummy = nlp.maskAtSign(vLine)
		
		vlOutput.append(' '.join(vLine.split(' ')))
	
	return '\n'.join(vlOutput) + '\n'
	


def convertSQuoteWSJ(pInput):
	'''
	Converts various types of single quotation marks in pInput to WSJ 
	style which is ` for opening and ' for closing instead of ' for both. 
	
	NOTE:
	- pInput must be tokenized.
	- For ', the heuristic is not perfect and fails if it is an opening 
	  quote and follows a token ending with s. In this case, it's assumed
	  as possession mark and is not converted.
	
	For ', where the opening and closing quotes are the same, the heuristic
	is that, a line is iterated for single quotations. The 1st, 3rd, 5th,
	and so forth (odds) are converted to ` and the 2nd, 4th, and so forth 
	(evens) to '. This assumes well-formed punctuation which may not be 
	the case for genres line UGC.
	'''
	
	vOutput = ""
	
	# replacing '
	vNextReplacement = "`"
	for i in range(len(pInput)):
		if pInput[i] == "'":
			## cheking if the quote is not possession or contraction mark
			## This STRONGLY assumes that the input is tokenized.
			if (i > 0 and pInput[i-1] != ' ') or (i < len(pInput) - 1 and pInput[i+1] != ' ') or (i > 1 and pInput[i-1] == ' ' and pInput[i-2] == 's'):
					vOutput += pInput[i]
					continue
			
			vOutput += vNextReplacement
			
			if vNextReplacement == "`":
				vNextReplacement = "'"
			elif vNextReplacement == "'":
				vNextReplacement = "`"
		else:
			vOutput += pInput[i]
	
	# replacing  ‘ and ’
	vPattern = re.compile(u"‘")
	vOutput = vPattern.sub("`", vOutput)
	vPattern = re.compile(u"’")
	vOutput = vPattern.sub("'", vOutput)
	
	return vOutput
	


def convertDQuoteWSJ(pInput):
	'''
	Converts various types of double quotation marks in pInput to WSJ 
	style which is `` for opening and '' for closing instead of " for both. 
	
	pInput should be tokenized and preferably a line. 
	
	For ", where the opening and closing quotes are the same, the heuristic
	is that, a line is iterated for single quotations. The 1st, 3rd, 5th,
	and so forth (odds) are converted to ` and the 2nd, 4th, and so forth 
	(evens) to '. This assumes well-formed punctuation which may not be 
	the case for genres line UGC.
	'''
	
	vOutput = ""
	
	# replacing "
	vNextReplacement = "``"
	for i in range(len(pInput)):
		if pInput[i] == '"':
			vOutput += vNextReplacement
			
			if vNextReplacement == "``":
				vNextReplacement = "''"
			elif vNextReplacement == "''":
				vNextReplacement = "``"
		else:
			vOutput += pInput[i]
	
	# replacing “ and ”
	vPattern = re.compile(u"“")
	vOutput = vPattern.sub("``", vOutput)
	vPattern = re.compile(u"”")
	vOutput = vPattern.sub("''", vOutput)
	
	return vOutput
	


def extractPOS(pTreebank):
	'''
	Given a Treebank, extract and returns POS tags.
	'''
	 
	vlPOSLines = []
	vPattern = re.compile(r"\(([^ ]+) ([^()]+)\)")
	
	for vTree in pTreebank.split('\n'):
		vlPOSTokens = []
		# masking token round brackets first, e.g. (NN )) to (NN -RRB)
		vTree = vTree.replace(" ))", " -RRB-)").replace(" ()", " -LRB-)")
		
		# extracting POS tags
		for vTokenPOS in re.findall(vPattern, vTree):
			vlPOSTokens.append(vTokenPOS[1] + '/' + vTokenPOS[0])
		
		vlPOSLines.append(" ".join(vlPOSTokens))
	
	return '\n'.join(vlPOSLines)
	 


def depCoNLL2PTBTree(pCoNLLFilename, pLanguage, pflgVerbos = True):
	'''
	Extracts dependency parses from CoNLL file and returns its representation 
	in PTB bracketing style.
	'''
	
	vCoNLLData = dloader.CoNLLDataLoader(pCoNLLFilename, pLanguage = pLanguage, pflgVerbos = pflgVerbos)
	
	vlPTBDepTrees = []
	for vSent in vCoNLLData.conllSentences:
		vlPTBDepTrees.append(vSent.depTree(pdAnnotationOptions = {"fine-or-coarse": "fine"}).getPTBTree())
	
	return '\n'.join(vlPTBDepTrees) + '\n'
	


def depPARC700ToPTBTree(pPARC700Filename, pflgVerbos = True):
	'''
	Extracts dependency parses from PARC 700 file and returns its representation 
	in PTB bracketing style.
	'''
	
	vPARC700Data = dloader.PARC700DataLoader(pPARC700Filename, pflgVerbos = pflgVerbos)
	
	vlPTBDepTrees = []
	for vSent in vPARC700Data.parcSentences:
		vlPTBDepTrees.append(vSent.depTree.getPTBTree())
	
	return '\n'.join(vlPTBDepTrees) + '\n'
	


def randomConstParse(pCorpus, pLang):
	'''
	Builds random constituency parse trees for sentences in pCorpus
	'''
	
	vParser = BConstParser()
	
	vlParseTrees = []
	for vSent in pCorpus.strip().split('\n'):
		vParseTree = vParser.randomParse(vSent).getPTBFormat()
		vlParseTrees.append("(TOP %s)" % vParseTree)
	
	return '\n'.join(vlParseTrees) + '\n'
	


